CREATE SCHEMA IF NOT EXISTS RESOURCE_MONITORING;
CREATE TABLE IF NOT EXISTS RESOURCE_MONITORING.SYS_INFO( machine_id VARCHAR(40), time TIMESTAMP, 
hostname VARCHAR(255), primary_ip VARCHAR(15), cpu_vendor VARCHAR(100), cpu_model VARCHAR(100), 
num_cores TINYINT(2), num_cpus TINYINT(2), cpu_mhz INT, cpu_cache_size BIGINT, os_name VARCHAR(255), 
os_version VARCHAR(100), default_gateway_ip VARCHAR(15), default_gateway_mac VARCHAR(20), 
active VARCHAR(10), memory_total BIGINT, swap_total BIGINT, PRIMARY KEY (machine_id));

CREATE TABLE IF NOT EXISTS RESOURCE_MONITORING.EVENTS( machine_id VARCHAR(40), label VARCHAR(32), time TIMESTAMP, attributes VARCHAR(64), router_mac VARCHAR(40), FOREIGN KEY (machine_id) REFERENCES SYS_INFO(machine_id));

CREATE TABLE IF NOT EXISTS RESOURCE_MONITORING.MACHINE_UTIL( machine_id  VARCHAR(40), start_ts TIMESTAMP, end_ts TIMESTAMP, measurements INT, 
cpu_user_avg DOUBLE, cpu_user_min DOUBLE, cpu_user_max DOUBLE, 
cpu_sys_avg DOUBLE, cpu_sys_min DOUBLE, cpu_sys_max DOUBLE, 
cpu_nice_avg DOUBLE, cpu_nice_min DOUBLE,cpu_nice_max DOUBLE,
cpu_idle_avg DOUBLE, cpu_idle_min DOUBLE, cpu_idle_max DOUBLE, 
cpu_wait_avg DOUBLE,  cpu_wait_min DOUBLE, cpu_wait_max DOUBLE, 
memory_used_avg BIGINT, memory_used_min BIGINT, memory_used_max BIGINT, 
memory_free_avg BIGINT, memory_free_min BIGINT, memory_free_max BIGINT, 
swap_used_avg BIGINT, swap_used_min BIGINT, swap_used_max BIGINT,
swap_free_avg BIGINT, swap_free_min BIGINT, swap_free_max BIGINT
, FOREIGN KEY (machine_id) REFERENCES SYS_INFO(machine_id));

CREATE TABLE IF NOT EXISTS RESOURCE_MONITORING.FILE_SYSTEM( machine_id  VARCHAR(40), start_ts TIMESTAMP, end_ts TIMESTAMP, measurements INT, 
fs_name VARCHAR(255), 
fs_location VARCHAR(255), 
fs_type VARCHAR(100), 
fs_size BIGINT, 
fs_free BIGINT, 
fs_used BIGINT, 
fs_files_avg BIGINT, fs_files_min BIGINT, fs_files_max BIGINT, 
fs_disk_reads_avg BIGINT, fs_disk_reads_min BIGINT, fs_disk_reads_max BIGINT, 
fs_disk_read_bytes_avg BIGINT,fs_disk_read_bytes_min BIGINT,fs_disk_read_bytes_max BIGINT,
fs_disk_writes_avg BIGINT, fs_disk_writes_min BIGINT, fs_disk_writes_max BIGINT, 
fs_disk_write_bytes_avg BIGINT, fs_disk_write_bytes_min BIGINT, fs_disk_write_bytes_max BIGINT
, FOREIGN KEY (machine_id) REFERENCES SYS_INFO(machine_id));

CREATE TABLE IF NOT EXISTS RESOURCE_MONITORING.PROCESS( machine_id VARCHAR(40), start_ts TIMESTAMP, end_ts TIMESTAMP, measurements INT, process_name VARCHAR(255),
process_start_time BIGINT, process_cpu_percent_avg DOUBLE, process_cpu_percent_min DOUBLE, process_cpu_percent_max DOUBLE, 
process_mem_avg BIGINT, process_mem_min BIGINT,process_mem_max BIGINT,
process_resident_avg BIGINT, process_resident_min BIGINT, process_resident_max BIGINT
, FOREIGN KEY (machine_id) REFERENCES SYS_INFO(machine_id));

CREATE TABLE IF NOT EXISTS RESOURCE_MONITORING.NETWORK( machine_id  VARCHAR(40), start_ts TIMESTAMP, end_ts TIMESTAMP, measurements INT, 
device_name VARCHAR(100),
ip_address VARCHAR(15), 
rx_bytes_avg BIGINT, rx_bytes_min BIGINT, rx_bytes_max BIGINT, 
rx_packets_avg BIGINT, rx_packets_min BIGINT, rx_packets_max BIGINT, 
rx_errors_avg BIGINT, rx_errors_min BIGINT, rx_errors_max BIGINT, 
rx_dropped_avg BIGINT, rx_dropped_min BIGINT, rx_dropped_max BIGINT, 
rx_overruns_avg BIGINT, rx_overruns_min BIGINT, rx_overruns_max BIGINT, 
tx_bytes_avg BIGINT, tx_bytes_min BIGINT, tx_bytes_max BIGINT, 
tx_packets_avg BIGINT, tx_packets_min BIGINT,tx_packets_max BIGINT,
tx_errors_avg BIGINT, tx_errors_min BIGINT, tx_errors_max BIGINT, 
tx_dropped_avg BIGINT, tx_dropped_min BIGINT, tx_dropped_max BIGINT, 
tx_overruns_avg BIGINT, tx_overruns_min BIGINT, tx_overruns_max BIGINT
, FOREIGN KEY (machine_id) REFERENCES SYS_INFO(machine_id));

CREATE TABLE IF NOT EXISTS RESOURCE_MONITORING.DISTRIBUTION2( machine_id  VARCHAR(40), start_ts TIMESTAMP, end_ts TIMESTAMP, measurements INT,  
instrument_name  VARCHAR(32), attributes  VARCHAR(40), high DOUBLE, medium_high DOUBLE, medium DOUBLE, medium_low DOUBLE, low DOUBLE, FOREIGN KEY (machine_id) REFERENCES SYS_INFO(machine_id));

CREATE TABLE IF NOT EXISTS RESOURCE_MONITORING.DISTRIBUTION( machine_id  VARCHAR(40), start_ts TIMESTAMP, end_ts TIMESTAMP, measurements INT,  
instrument_name  VARCHAR(32), attributes  VARCHAR(40), value_stream VARCHAR(40), FOREIGN KEY (machine_id) REFERENCES SYS_INFO(machine_id));


CREATE TABLE IF NOT EXISTS RESOURCE_MONITORING.LATENCY_AND_BANDWIDTH( sender_machine_id  VARCHAR(40), time TIMESTAMP, sender_device_name VARCHAR(100), sender_gateway VARCHAR(15), 
receiver_machine_id  VARCHAR(40), receiver_device_name VARCHAR(100), receiver_gateway VARCHAR(15), latency DOUBLE, bandwidth DOUBLE, FOREIGN KEY (sender_machine_id) REFERENCES SYS_INFO(machine_id), FOREIGN KEY (receiver_machine_id) REFERENCES SYS_INFO(machine_id));

SET @@global.sql_mode = 'ALLOW_INVALID_DATES';