CREATE SCHEMA IF NOT EXISTS VIEWS;

CREATE VIEW IF NOT EXISTS views.cpu_avg_working_utilization AS (SELECT (1 - AVG(cpu_idle_avg)) as avg_cpu_util, count(CPU_IDLE_AVG) as measurements FROM RESOURCE_MONITORING.MACHINE_UTIL WHERE (extract(hour from start_ts) >= 9 AND extract(hour from start_ts) <= 17));
CREATE VIEW IF NOT EXISTS views.cpu_avg_nightly_utilization AS (SELECT (1 - AVG(cpu_idle_avg)) as avg_cpu_util, count(CPU_IDLE_AVG) as measurements FROM RESOURCE_MONITORING.MACHINE_UTIL WHERE (extract(hour from start_ts) > 17 OR extract(hour from start_ts) < 9));

CREATE VIEW IF NOT EXISTS views.cpu_avg_hourly_utilization AS (SELECT extract(hour from start_ts) AS hour, count(CPU_IDLE_AVG) as measurements, (1 - AVG(CPU_IDLE_AVG)) AS cpu_util_avg FROM RESOURCE_MONITORING.MACHINE_UTIL GROUP BY extract(hour from start_ts));
CREATE VIEW IF NOT EXISTS views.cpu_avg_daily_utilization AS (SELECT DAYNAME (start_ts) AS day, count(CPU_IDLE_AVG) as measurements, (1 - AVG(CPU_IDLE_AVG)) AS cpu_util_avg FROM RESOURCE_MONITORING.MACHINE_UTIL GROUP BY DAYNAME (start_ts));


CREATE VIEW IF NOT EXISTS views.cpu_avg_weekend_utilization AS (SELECT (1 - AVG(cpu_idle_avg)) as avg_cpu_util, count(CPU_IDLE_AVG) as measurements  FROM RESOURCE_MONITORING.MACHINE_UTIL WHERE (DAY_OF_WEEK (start_ts) = 1 OR DAY_OF_WEEK (start_ts) = 7));
CREATE VIEW IF NOT EXISTS views.cpu_avg_weekday_utilization AS (SELECT (1 - AVG(cpu_idle_avg)) as avg_cpu_util, count(CPU_IDLE_AVG) as measurements  FROM RESOURCE_MONITORING.MACHINE_UTIL WHERE (DAY_OF_WEEK (start_ts) > 1 AND DAY_OF_WEEK (start_ts) < 7));



CREATE VIEW IF NOT EXISTS views.probability_by_machine AS (
SELECT hostname, instrument_name, min(start_ts) as start_time, max(end_ts) as end_time, sum(measurements) as total_measurements,  AVG(high), AVG(medium_high), AVG(medium), AVG(medium_low), AVG(low)  FROM RESOURCE_MONITORING.SYS_INFO, RESOURCE_MONITORING.DISTRIBUTION 
WHERE RESOURCE_MONITORING.SYS_INFO.machine_id = RESOURCE_MONITORING.DISTRIBUTION.machine_id
GROUP BY (RESOURCE_MONITORING.DISTRIBUTION.machine_id, instrument_name)
);

CREATE VIEW IF NOT EXISTS views.fs_hourly_usage AS (
SELECT 
machine_id,
hour(start_ts) as hour,
sum(measurements) as measurements,
fs_name, 
fs_type,
fs_location,
max(fs_size) as fs_size,
min(fs_free) as fs_free, max(fs_used) as fs_used, 
avg(fs_files_avg) as fs_files_avg, min(fs_files_min) as fs_files_min, max(fs_files_max) as fs_files_max, 
avg(fs_disk_reads_avg) as fs_disk_reads_avg, min(fs_disk_reads_min) as fs_disk_reads_min, max(fs_disk_reads_max) as fs_disk_reads_max, 
avg(fs_disk_read_bytes_avg) as fs_disk_read_bytes_avg, min(fs_disk_read_bytes_min) as fs_disk_read_bytes_min, max(fs_disk_read_bytes_max) as fs_disk_read_bytes_max, 
avg(fs_disk_writes_avg) as fs_disk_writes_avg, min(fs_disk_writes_min) as fs_disk_writes_min, max(fs_disk_writes_max) as fs_disk_writes_max, 
avg(fs_disk_write_bytes_avg) as fs_disk_write_bytes_avg, min(fs_disk_write_bytes_min) as fs_disk_write_bytes_min, max(fs_disk_write_bytes_max) as fs_disk_write_bytes_max 
FROM resource_monitoring.file_system 
GROUP BY machine_id, fs_name, hour(start_ts)
);

CREATE VIEW IF NOT EXISTS views.fs_last_five_min_usage AS (
SELECT 
machine_id,
min(start_ts) as start_ts, 
sum(measurements) as measurements,
fs_name, 
fs_type,
fs_location,
max(fs_size) as fs_size,
min(fs_free) as fs_free, max(fs_used) as fs_used, 
avg(fs_files_avg) as fs_files_avg, min(fs_files_min) as fs_files_min, max(fs_files_max) as fs_files_max, 
avg(fs_disk_reads_avg) as fs_disk_reads_avg, min(fs_disk_reads_min) as fs_disk_reads_min, max(fs_disk_reads_max) as fs_disk_reads_max, 
avg(fs_disk_read_bytes_avg) as fs_disk_read_bytes_avg, min(fs_disk_read_bytes_min) as fs_disk_read_bytes_min, max(fs_disk_read_bytes_max) as fs_disk_read_bytes_max, 
avg(fs_disk_writes_avg) as fs_disk_writes_avg, min(fs_disk_writes_min) as fs_disk_writes_min, max(fs_disk_writes_max) as fs_disk_writes_max, 
avg(fs_disk_write_bytes_avg) as fs_disk_write_bytes_avg, min(fs_disk_write_bytes_min) as fs_disk_write_bytes_min, max(fs_disk_write_bytes_max) as fs_disk_write_bytes_max 
FROM resource_monitoring.file_system 
WHERE   start_ts >= DATEADD('MINUTE', -5, NOW())
GROUP BY machine_id, fs_name
);

CREATE VIEW IF NOT EXISTS views.fs_last_thirty_min_usage AS (
SELECT 
machine_id,
min(start_ts) as start_ts, 
sum(measurements) as measurements,
fs_name, 
fs_type,
fs_location,
max(fs_size) as fs_size,
min(fs_free) as fs_free, max(fs_used) as fs_used, 
avg(fs_files_avg) as fs_files_avg, min(fs_files_min) as fs_files_min, max(fs_files_max) as fs_files_max, 
avg(fs_disk_reads_avg) as fs_disk_reads_avg, min(fs_disk_reads_min) as fs_disk_reads_min, max(fs_disk_reads_max) as fs_disk_reads_max, 
avg(fs_disk_read_bytes_avg) as fs_disk_read_bytes_avg, min(fs_disk_read_bytes_min) as fs_disk_read_bytes_min, max(fs_disk_read_bytes_max) as fs_disk_read_bytes_max, 
avg(fs_disk_writes_avg) as fs_disk_writes_avg, min(fs_disk_writes_min) as fs_disk_writes_min, max(fs_disk_writes_max) as fs_disk_writes_max, 
avg(fs_disk_write_bytes_avg) as fs_disk_write_bytes_avg, min(fs_disk_write_bytes_min) as fs_disk_write_bytes_min, max(fs_disk_write_bytes_max) as fs_disk_write_bytes_max 
FROM resource_monitoring.file_system 
WHERE   start_ts >= DATEADD('MINUTE', -30, NOW())
GROUP BY machine_id, fs_name, fs_location
);

CREATE VIEW IF NOT EXISTS views.fs_last_day_usage AS (
SELECT 
machine_id,
min(start_ts) as start_ts, 
sum(measurements) as measurements,
fs_name, 
fs_type,
fs_location,
max(fs_size) as fs_size,
min(fs_free) as fs_free, max(fs_used) as fs_used, 
avg(fs_files_avg) as fs_files_avg, min(fs_files_min) as fs_files_min, max(fs_files_max) as fs_files_max, 
avg(fs_disk_reads_avg) as fs_disk_reads_avg, min(fs_disk_reads_min) as fs_disk_reads_min, max(fs_disk_reads_max) as fs_disk_reads_max, 
avg(fs_disk_read_bytes_avg) as fs_disk_read_bytes_avg, min(fs_disk_read_bytes_min) as fs_disk_read_bytes_min, max(fs_disk_read_bytes_max) as fs_disk_read_bytes_max, 
avg(fs_disk_writes_avg) as fs_disk_writes_avg, min(fs_disk_writes_min) as fs_disk_writes_min, max(fs_disk_writes_max) as fs_disk_writes_max, 
avg(fs_disk_write_bytes_avg) as fs_disk_write_bytes_avg, min(fs_disk_write_bytes_min) as fs_disk_write_bytes_min, max(fs_disk_write_bytes_max) as fs_disk_write_bytes_max 
FROM resource_monitoring.file_system 
WHERE   start_ts >= DATEADD('DAY', -1, NOW())
GROUP BY machine_id, fs_name, fs_location
);