/***************************************************************************
 *                                                                         *
 * numonic Library                                                         *
 * Copyright (C) 2009-2011 Distributed Systems Architecture Research Group *
 * University of St Andrews, Scotland                                      *
 * http://blogs.cs.st-andrews.ac.uk/numonic                                    *
 *                                                                         *
 * This file is part of remote_management, a package providing             *
 * functionality for remotely managing a specified application.            *
 *                                                                         *
 * numonic is free software: you can redistribute it and/or                *
 * modify it under the terms of the GNU General Public License as          *
 * published by the Free Software Foundation, either version 3 of the      *
 * License, or (at your option) any later version.                         *
 *                                                                         *
 * numonic is distributed in the hope that it will be useful,              *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 * GNU General Public License for more details.                            *
 *                                                                         *
 * You should have received a copy of the GNU General Public License       *
 * along with numonic.  If not, see <http://www.gnu.org/licenses/>.        *
 *                                                                         *
 ***************************************************************************/
package uk.ac.standrews.cs.numonic.aggregation;

import uk.ac.standrews.cs.nds.util.Diagnostic;
import uk.ac.standrews.cs.nds.util.DiagnosticLevel;
import uk.ac.standrews.cs.numonic.util.Time;

/**
 * Thread that periodically calculates aggregations.
 *
 * @author Alan Dearle (al@cs.st-andrews.ac.uk)
 */
public class AggregationMeasurements extends Thread {

    private final long time_between_aggregations;
    private final Availability availability;

    private static final long DEFAULT_TIME_BETWEEN_AGGREGATIONS = Time.ONE_HOUR_IN_MS;

    // -------------------------------------------------------------------------------------------------------

    /**
     * Initializes thread with aggregation interval of one hour.
     */
    public AggregationMeasurements() {

        this(DEFAULT_TIME_BETWEEN_AGGREGATIONS);
    }

    /**
     * Initializes thread with specified aggregation interval.
     * @param time_between_aggregations the aggregation interval
     */
    public AggregationMeasurements(final long time_between_aggregations) {

        this.time_between_aggregations = time_between_aggregations;
        availability = new Availability();
    }

    // -------------------------------------------------------------------------------------------------------

    @Override
    public synchronized void run() {

        while (true) {
            try {
                availability.calculateAggregate();
                wait(time_between_aggregations);
            }
            catch (final Exception e) {
                Diagnostic.trace(DiagnosticLevel.FULL, "Exception running AgregationMeasurements: " + e.getMessage());
                return;
            }
        }
    }
}
