/***************************************************************************
 *                                                                         *
 * numonic Library                                                         *
 * Copyright (C) 2009-2010 Distributed Systems Architecture Research Group *
 * University of St Andrews, Scotland                                      *
 * http://blogs.cs.st-andrews.ac.uk/numonic                                    *
 *                                                                         *
 * This file is part of remote_management, a package providing             *
 * functionality for remotely managing a specified application.            *
 *                                                                         *
 * numonic is free software: you can redistribute it and/or                *
 * modify it under the terms of the GNU General Public License as          *
 * published by the Free Software Foundation, either version 3 of the      *
 * License, or (at your option) any later version.                         *
 *                                                                         *
 * numonic is distributed in the hope that it will be useful,              *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 * GNU General Public License for more details.                            *
 *                                                                         *
 * You should have received a copy of the GNU General Public License       *
 * along with numonic.  If not, see <http://www.gnu.org/licenses/>.        *
 *                                                                         *
 ***************************************************************************/
package uk.ac.standrews.cs.numonic.classify;

import java.util.Date;

import uk.ac.standrews.cs.numonic.distribution.Distribution;

/**
 * This class is used as the base class for all classifiers.
 * Provides a general purpose classification method.
 *
 * @author Alan Dearle (al@cs.st-andrews.ac.uk)
 */
public abstract class ClassifierBase {

    static final double[] PERCENTAGE_THRESHOLDS = new double[]{0.8, 0.6, 0.4, 0.2};
    static final int MILLI_SECONDS_PER_SECOND = 1000;

    static final double THRESHOLD_FACTOR_1 = 0.8;
    static final double THRESHOLD_FACTOR_2 = 0.6;
    static final double THRESHOLD_FACTOR_3 = 0.4;
    static final double THRESHOLD_FACTOR_4 = 0.2;

    /**
     * Classifies a measurement into a distribution of {H,MH,M,ML,L}.
     * This is done so that average distributions can be created by averaging individual classifications.
     *
     * @param instrument_name the instrument being classified
     * @param attributes any attributes associated with the instrument
     * @param time the time at which the measurement was made
     * @param value the value being classified
     * @param thresholds the thresholds making classification
     * @return the distribution
     */
    protected Distribution classify(final String instrument_name, final String attributes, final Date time, final double value, final double[] thresholds) {

        if (value > thresholds[0]) { return new Distribution(instrument_name, attributes, time, 1.0f, 0.0f, 0.0f, 0.0f, 0.0f); }
        if (value > thresholds[1]) { return new Distribution(instrument_name, attributes, time, 0.0f, 1.0f, 0.0f, 0.0f, 0.0f); }
        if (value > thresholds[2]) { return new Distribution(instrument_name, attributes, time, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f); }
        if (value > thresholds[3]) { return new Distribution(instrument_name, attributes, time, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f); }

        return new Distribution(instrument_name, attributes, time, 0.0f, 0.0f, 0.0f, 0.0f, 1.0f);
    }
}
