/***************************************************************************
 *                                                                         *
 * numonic Library                                                         *
 * Copyright (C) 2009-2011 Distributed Systems Architecture Research Group *
 * University of St Andrews, Scotland                                      *
 * http://blogs.cs.st-andrews.ac.uk/numonic                                    *
 *                                                                         *
 * This file is part of remote_management, a package providing             *
 * functionality for remotely managing a specified application.            *
 *                                                                         *
 * numonic is free software: you can redistribute it and/or                *
 * modify it under the terms of the GNU General Public License as          *
 * published by the Free Software Foundation, either version 3 of the      *
 * License, or (at your option) any later version.                         *
 *                                                                         *
 * numonic is distributed in the hope that it will be useful,              *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 * GNU General Public License for more details.                            *
 *                                                                         *
 * You should have received a copy of the GNU General Public License       *
 * along with numonic.  If not, see <http://www.gnu.org/licenses/>.        *
 *                                                                         *
 ***************************************************************************/

package uk.ac.standrews.cs.numonic.classify;

import uk.ac.standrews.cs.numonic.data.MachineUtilisationData;
import uk.ac.standrews.cs.numonic.distribution.Distribution;

/**
 * Classifier for machine utilisation data.
 *
 * @author Alan Dearle (al@cs.st-andrews.ac.uk)
 */
public class MachineUtilisationClassifier extends ClassifierBase implements IClassifier<MachineUtilisationData> {

    @Override
    public Distribution[] classify(final MachineUtilisationData data) {

        return new Distribution[]{classify("cpu", "", data.getDate(), 1 - data.getCpuIdleTotal(), PERCENTAGE_THRESHOLDS), classify("memory", "", data.getDate(), percentageUsed(data.getMemoryUsed(), data.getMemoryFree()), PERCENTAGE_THRESHOLDS),
                        classify("swap", "", data.getDate(), percentageUsed(data.getSwapUsed(), data.getSwapFree()), PERCENTAGE_THRESHOLDS)};
    }

    /**
     * Returns the percentage used.
     * @return the % of used / ( used + free ) in the range of 0-1 i.e. 95% is return as 0.95
     */
    private double percentageUsed(final long used, final long free) {

        return used / (double) (used + free);
    }

    @Override
    public MachineUtilisationData getContentType() {

        return new MachineUtilisationData().getNewInstance();
    }
}
