/***************************************************************************
 *                                                                         *
 * numonic Library                                                         *
 * Copyright (C) 2009-2011 Distributed Systems Architecture Research Group *
 * University of St Andrews, Scotland                                      *
 * http://blogs.cs.st-andrews.ac.uk/numonic                                    *
 *                                                                         *
 * This file is part of remote_management, a package providing             *
 * functionality for remotely managing a specified application.            *
 *                                                                         *
 * numonic is free software: you can redistribute it and/or                *
 * modify it under the terms of the GNU General Public License as          *
 * published by the Free Software Foundation, either version 3 of the      *
 * License, or (at your option) any later version.                         *
 *                                                                         *
 * numonic is distributed in the hope that it will be useful,              *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 * GNU General Public License for more details.                            *
 *                                                                         *
 * You should have received a copy of the GNU General Public License       *
 * along with numonic.  If not, see <http://www.gnu.org/licenses/>.        *
 *                                                                         *
 ***************************************************************************/

package uk.ac.standrews.cs.numonic.data;

/**
 * Used to store network usage data.
 *
 * @author Alan Dearle (al@cs.st-andrews.ac.uk)
 */
public class NetworkData extends Data implements IData<NetworkData> {

    public String device_name; // Name of the device
    public String ip_address; // IP address of the device
    public long rx_bytes; // Total bytes received by the interface
    public long rx_packets; // Total packets received by the interface
    public long rx_errors; // Total number of errors made whist receiving by the interface
    public long rx_dropped; // Total number of packets dropped whilst receiving by the interface
    public long rx_overruns; // Total number of overruns whilst receiving by the interface
    public long tx_bytes; // Total bytes transmitted by the interface
    public long tx_packets; // Total packets transmitted by the interface
    public long tx_errors; // Total number of errors made whist transmitting by the interface
    public long tx_dropped; // Total number of packets dropped whilst transmitting by the interface
    public long tx_overruns; // Total number of overruns whilst transmitting by the interface

    @Override
    public String toString() {

        final StringBuilder stringBuilder = new StringBuilder();
        stringBuilder.append("Network Monitor Results [Device_name=");
        stringBuilder.append(device_name);
        stringBuilder.append("\n");
        stringBuilder.append("ip_address=");
        stringBuilder.append(ip_address);
        stringBuilder.append("\n");
        stringBuilder.append("rx_bytes=");
        stringBuilder.append(rx_bytes);
        stringBuilder.append("\n");
        stringBuilder.append("rx_packets=");
        stringBuilder.append(rx_packets);
        stringBuilder.append("\n");
        stringBuilder.append("rx_errors=");
        stringBuilder.append(rx_errors);
        stringBuilder.append("\n");
        stringBuilder.append("rx_dropped=");
        stringBuilder.append(rx_dropped);
        stringBuilder.append("\n");
        stringBuilder.append("rx_overruns=");
        stringBuilder.append(rx_overruns);
        stringBuilder.append("\n");
        stringBuilder.append("tx_bytes=");
        stringBuilder.append(tx_bytes);
        stringBuilder.append("\n");
        stringBuilder.append("tx_packets=");
        stringBuilder.append(tx_packets);
        stringBuilder.append("\n");
        stringBuilder.append("tx_errors=");
        stringBuilder.append(tx_errors);
        stringBuilder.append("\n");
        stringBuilder.append("tx_dropped=");
        stringBuilder.append(tx_dropped);
        stringBuilder.append("\n");
        stringBuilder.append("tx_overruns=");
        stringBuilder.append(tx_overruns);
        stringBuilder.append("]");
        return stringBuilder.toString();
    }

    /**
     * Returns a new object in which each numerical field contains the minimum value from this object and the other given object.
     * Other fields are taken from the other object.
     *
     * @param other the other object
     * @return an object containing the minimum values
     */
    @Override
    public NetworkData min(final NetworkData other) {

        final NetworkData min = new NetworkData();

        min.device_name = other.device_name;
        min.ip_address = other.ip_address;

        min.rx_bytes = min(rx_bytes, other.rx_bytes);
        min.rx_packets = min(rx_packets, other.rx_packets);
        min.rx_errors = min(rx_errors, other.rx_errors);
        min.rx_dropped = min(rx_dropped, other.rx_dropped);
        min.rx_overruns = min(rx_overruns, other.rx_overruns);
        min.tx_bytes = min(tx_bytes, other.tx_bytes);
        min.tx_packets = min(tx_packets, other.tx_packets);
        min.tx_errors = min(tx_errors, other.tx_errors);

        return min;
    }

    /**
     * Returns a new object in which each numerical field contains the maximum value from this object and the other given object.
     * Other fields are taken from the other object.
     *
     * @param other the other object
     * @return an object containing the maximum values
     */
    @Override
    public NetworkData max(final NetworkData other) {

        final NetworkData max = new NetworkData();

        max.device_name = other.device_name;
        max.ip_address = other.ip_address;

        max.rx_bytes = max(rx_bytes, other.rx_bytes);
        max.rx_packets = max(rx_packets, other.rx_packets);
        max.rx_errors = max(rx_errors, other.rx_errors);
        max.rx_dropped = max(rx_dropped, other.rx_dropped);
        max.rx_overruns = max(rx_overruns, other.rx_overruns);
        max.tx_bytes = max(tx_bytes, other.tx_bytes);
        max.tx_packets = max(tx_packets, other.tx_packets);
        max.tx_errors = max(tx_errors, other.tx_errors);

        return max;
    }

    /**
     * Returns a new object in which each numerical field contains the sum of the values from this object and the other given object.
     * Other fields are taken from the other object.
     *
     * @param other the other object
     * @return an object containing the summed values
     */
    @Override
    public NetworkData sum(final NetworkData other) {

        final NetworkData total = new NetworkData();

        total.device_name = other.device_name;
        total.ip_address = other.ip_address;

        total.rx_bytes = rx_bytes + other.rx_bytes;
        total.rx_packets = rx_packets + other.rx_packets;
        total.rx_errors = rx_errors + other.rx_errors;
        total.rx_dropped = rx_dropped + other.rx_dropped;
        total.rx_overruns = rx_overruns + other.rx_overruns;
        total.tx_bytes = tx_bytes + other.tx_bytes;
        total.tx_packets = tx_packets + other.tx_packets;
        total.tx_errors = tx_errors + other.tx_errors;

        return total;
    }

    /**
     * Returns a new object in which each numerical field contains the value from this object divided by the given number of measurements.
     * Other fields are taken from the other object.
     *
     * @param numberOfMeasurements the divisor
     * @return an object containing the converted values
     */
    @Override
    public NetworkData convertToAverage(final int numberOfMeasurements) {

        final NetworkData average = new NetworkData();

        average.device_name = device_name;
        average.ip_address = ip_address;

        average.rx_bytes = rx_bytes / numberOfMeasurements;
        average.rx_packets = rx_packets / numberOfMeasurements;
        average.rx_errors = rx_errors / numberOfMeasurements;
        average.rx_dropped = rx_dropped / numberOfMeasurements;
        average.rx_overruns = rx_overruns / numberOfMeasurements;
        average.tx_bytes = tx_bytes / numberOfMeasurements;
        average.tx_packets = tx_packets / numberOfMeasurements;
        average.tx_errors = tx_errors / numberOfMeasurements;

        return average;
    }

    /**
     * Returns a new object in which each numerical field contains the value from another given object subtracted from the value from this object.
     * Other fields are taken from the other object.
     *
     * @param base the other object
     * @return an object containing the adjusted values
     */
    @Override
    public NetworkData adjustForBase(final NetworkData base) {

        final NetworkData adjusted = new NetworkData();

        adjusted.device_name = device_name;
        adjusted.ip_address = ip_address;

        adjusted.rx_overruns = rx_overruns;

        adjusted.rx_bytes = rx_bytes - base.rx_bytes;
        adjusted.rx_packets = rx_packets - base.rx_packets;
        adjusted.rx_errors = rx_errors - base.rx_errors;
        adjusted.rx_dropped = rx_dropped - base.rx_dropped;

        adjusted.tx_bytes = tx_bytes - base.tx_bytes;
        adjusted.tx_packets = tx_packets - base.tx_packets;
        adjusted.tx_errors = tx_errors - base.tx_errors;

        return adjusted;
    }

    /**
     * Returns a new instance.
     * @return a new instance
     */
    @Override
    public NetworkData getNewInstance() {

        return new NetworkData();
    }

    /**
     * Whether this dataset has any data in it that needs to be adjusted for the base - whether the {@link IData#adjustForBase(Object)}
     * method needs to be called.
     *
     * @return true if the {@link IData#adjustForBase(Object)} method does any work.
     */
    @Override
    public boolean needsAdjustedForBase() {

        return true;
    }

    /**
     * Returns the device name.
     * @return the device name
     */
    public String getDeviceName() {

        return device_name;
    }

    /**
     * Returns the IP address.
     * @return the IP address
     */
    public String getIPAddress() {

        return ip_address;
    }

    /**
     * Returns the bytes received.
     * @return the bytes received
     */
    public long getRXBytes() {

        return rx_bytes;
    }

    /**
     * Returns the packets received.
     * @return the packets received
     */
    public long getRXPackets() {

        return rx_packets;
    }

    /**
     * Returns the reception errors.
     * @return the reception errors
     */
    public long getRXErrors() {

        return rx_errors;
    }

    /**
     * Returns the reception drop count.
     * @return the reception drop count
     */
    public long getRXDropped() {

        return rx_dropped;
    }

    /**
     * Returns the reception overrun count.
     * @return the reception overrun count
     */
    public long getRXOverruns() {

        return rx_overruns;
    }

    /**
     * Returns the bytes transmitted.
     * @return the bytes transmitted
     */
    public long getTXBytes() {

        return tx_bytes;
    }

    /**
     * Returns the packets transmitted.
     * @return the packets transmitted
     */
    public long getTXPackets() {

        return tx_packets;
    }

    /**
     * Returns the transmission errors.
     * @return the transmission errors
     */
    public long getTXErrors() {

        return tx_errors;
    }

    /**
     * Returns the transmission drop count.
     * @return the transmission drop count
     */
    public long getTXDropped() {

        return tx_dropped;
    }

    /**
     * Returns the transmission overrun count.
     * @return the transmission overrun count
     */
    public long getTXOverruns() {

        return tx_overruns;
    }

    /**
     * Sets the device name.
     * @param device_name the device name
     */
    public void setDeviceName(final String device_name) {

        this.device_name = device_name;
    }

    /**
     * Sets the IP address.
     * @param ip_address the IP address
     */
    public void setIPAddress(final String ip_address) {

        this.ip_address = ip_address;
    }

    /**
     * Sets the bytes received.
     * @param rx_bytes the bytes received
     */
    public void setRXBytes(final long rx_bytes) {

        this.rx_bytes = rx_bytes;
    }

    /**
     * Sets the packets received.
     * @param rx_packets the packets received
     */
    public void setRXPackets(final long rx_packets) {

        this.rx_packets = rx_packets;
    }

    /**
     * Sets the reception errors.
     * @param rx_errors the reception errors
     */
    public void setRXErrors(final long rx_errors) {

        this.rx_errors = rx_errors;
    }

    /**
     * Sets the reception overrun count.
     * @param rx_overruns the reception overrun count
     */
    public void setRXOverruns(final long rx_overruns) {

        this.rx_overruns = rx_overruns;
    }

    /**
     * Sets the reception drop count.
     * @param rx_dropped the reception drop count
     */
    public void setRXDropped(final long rx_dropped) {

        this.rx_dropped = rx_dropped;
    }

    /**
     * Sets the bytes transmitted.
     * @param tx_bytes the bytes transmitted
     */
    public void setTXBytes(final long tx_bytes) {

        this.tx_bytes = tx_bytes;
    }

    /**
     * Sets the packets transmitted.
     * @param tx_packets the packets transmitted
     */
    public void setTXPackets(final long tx_packets) {

        this.tx_packets = tx_packets;
    }

    /**
     * Sets the transmission errors.
     * @param tx_errors the transmission errors
     */
    public void setTXErrors(final long tx_errors) {

        this.tx_errors = tx_errors;
    }

    /**
     * Sets the transmission drop count.
     * @param tx_dropped the transmission drop count
     */
    public void setTXDropped(final long tx_dropped) {

        this.tx_dropped = tx_dropped;
    }

    /**
     * Sets the transmission overrun count.
     * @param tx_overruns the transmission overrun count
     */
    public void setTXOverruns(final long tx_overruns) {

        this.tx_overruns = tx_overruns;
    }
}
