/***************************************************************************
 *                                                                         *
 * numonic Library                                                         *
 * Copyright (C) 2009-2011 Distributed Systems Architecture Research Group *
 * University of St Andrews, Scotland                                      *
 * http://blogs.cs.st-andrews.ac.uk/numonic                                    *
 *                                                                         *
 * This file is part of remote_management, a package providing             *
 * functionality for remotely managing a specified application.            *
 *                                                                         *
 * numonic is free software: you can redistribute it and/or                *
 * modify it under the terms of the GNU General Public License as          *
 * published by the Free Software Foundation, either version 3 of the      *
 * License, or (at your option) any later version.                         *
 *                                                                         *
 * numonic is distributed in the hope that it will be useful,              *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 * GNU General Public License for more details.                            *
 *                                                                         *
 * You should have received a copy of the GNU General Public License       *
 * along with numonic.  If not, see <http://www.gnu.org/licenses/>.        *
 *                                                                         *
 ***************************************************************************/
package uk.ac.standrews.cs.numonic.event;

import java.util.ArrayList;
import java.util.List;

/**
 * Records a sequence of events.
 *
 * @author Alan Dearle (al@cs.st-andrews.ac.uk)
 */
public class EventCollector {

    private final List<Event> events;
    private final int measurements_before_summarization;

    // -------------------------------------------------------------------------------------------------------

    /**
     * Initializes the event collector.
     * @param measurements_before_summarization the capacity of the collector before becoming full
     */
    public EventCollector(final int measurements_before_summarization) {

        events = new ArrayList<Event>();
        this.measurements_before_summarization = measurements_before_summarization;
    }

    // -------------------------------------------------------------------------------------------------------

    /**
     * Returns true if the collector is full.
     * @return true if the collector is full
     */
    public boolean isFull() {

        return events.size() >= measurements_before_summarization;
    }

    /**
     * Returns true if the collector is empty.
     * @return true if the collector is empty
     */
    public boolean isEmpty() {

        return events.isEmpty();
    }

    /**
     * Discards all recored events.
     */
    public void reset() {

        events.clear();
    }

    /**
     * Adds a new event.
     * @param e the event
     */
    public void addEvent(final Event e) {

        events.add(e);
    }

    /**
     * Returns the recorded events.
     * @return the recorded events
     */
    public List<Event> getEvents() {

        return events;
    }
}
