package uk.ac.standrews.cs.numonic.main;

import java.lang.management.ManagementFactory;
import java.lang.management.ThreadMXBean;

public class ThreadLister {

    static ThreadGroup rootThreadGroup = null;

    static ThreadGroup getRootThreadGroup() {

        if (rootThreadGroup != null) { return rootThreadGroup; }
        ThreadGroup tg = Thread.currentThread().getThreadGroup();
        ThreadGroup ptg;
        while ((ptg = tg.getParent()) != null) {
            tg = ptg;
        }
        return tg;
    }

    static ThreadGroup[] getAllThreadGroups() {

        final ThreadGroup root = getRootThreadGroup();
        int nAlloc = root.activeGroupCount();
        int n = 0;
        ThreadGroup[] groups;
        do {
            nAlloc *= 2;
            groups = new ThreadGroup[nAlloc];
            n = root.enumerate(groups, true);
        }
        while (n == nAlloc);

        final ThreadGroup[] allGroups = new ThreadGroup[n + 1];
        allGroups[0] = root;
        System.arraycopy(groups, 0, allGroups, 1, n);
        return allGroups;
    }

    static ThreadGroup getThreadGroup(final String name) {

        if (name == null) { throw new NullPointerException("Null name"); }
        final ThreadGroup[] groups = getAllThreadGroups();
        for (final ThreadGroup group : groups) {
            if (group.getName().equals(name)) { return group; }
        }
        return null;
    }

    /** Find the root thread group and list it recursively */
    static Thread[] getAllThreads() {

        final ThreadGroup root = getRootThreadGroup();
        final ThreadMXBean thbean = ManagementFactory.getThreadMXBean();
        int nAlloc = thbean.getThreadCount();
        int n = 0;
        Thread[] threads;
        do {
            nAlloc *= 2;
            threads = new Thread[nAlloc];
            n = root.enumerate(threads, true);
        }
        while (n == nAlloc);
        return java.util.Arrays.copyOf(threads, n);
    }

    public static void main(final String[] args) {

        System.out.println(ThreadLister.getAllThreads().toString());
    }
}
