/***************************************************************************
 *                                                                         *
 * numonic Library                                                         *
 * Copyright (C) 2009-2011 Distributed Systems Architecture Research Group *
 * University of St Andrews, Scotland                                      *
 * http://blogs.cs.st-andrews.ac.uk/numonic                                    *
 *                                                                         *
 * This file is part of remote_management, a package providing             *
 * functionality for remotely managing a specified application.            *
 *                                                                         *
 * numonic is free software: you can redistribute it and/or                *
 * modify it under the terms of the GNU General Public License as          *
 * published by the Free Software Foundation, either version 3 of the      *
 * License, or (at your option) any later version.                         *
 *                                                                         *
 * numonic is distributed in the hope that it will be useful,              *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 * GNU General Public License for more details.                            *
 *                                                                         *
 * You should have received a copy of the GNU General Public License       *
 * along with numonic.  If not, see <http://www.gnu.org/licenses/>.        *
 *                                                                         *
 ***************************************************************************/

package uk.ac.standrews.cs.numonic.monitor;

import org.hyperic.sigar.CpuPerc;
import org.hyperic.sigar.Mem;
import org.hyperic.sigar.SigarException;
import org.hyperic.sigar.Swap;

import uk.ac.standrews.cs.numonic.data.IData;
import uk.ac.standrews.cs.numonic.data.MachineUtilisationData;

/**
 * Monitor for machine utilisation.
 *
 * @author Alan Dearle (al@cs.st-andrews.ac.uk)
 */
public class MachineUtilisationMonitor extends MonitorBase {

    /**
     * Returns machine utilisation data.
     *
     * @return machine utilisation data
     * @throws SigarException if an error occurs accessing sigar information
     */
    public MachineUtilisationData getInfo() throws SigarException {

        final CpuPerc cpu = sigar.getCpuPerc();
        final Mem mem = sigar.getMem();
        final Swap swap = sigar.getSwap();

        return new MachineUtilisationData(cpu.getUser(), cpu.getSys(), cpu.getIdle(), cpu.getWait(), cpu.getNice(), convertBytesToKBytes(mem.getUsed()), convertBytesToKBytes(mem.getFree()), convertBytesToKBytes(swap.getUsed()), convertBytesToKBytes(swap.getFree()));
    }

    // -------------------------------------------------------------------------------------------------------

    /**
     * Prints out monitoring data.
     *
     * @param args ignored
     * @throws SigarException if an error occurs accessing sigar information
     */
    public static void main(final String[] args) throws SigarException {

        final MachineUtilisationMonitor mu = new MachineUtilisationMonitor();
        final IData<MachineUtilisationData> results = mu.getInfo();
        System.out.println(results);
    }
}
