/***************************************************************************
 *                                                                         *
 * numonic Library                                                         *
 * Copyright (C) 2009-2011 Distributed Systems Architecture Research Group *
 * University of St Andrews, Scotland                                      *
 * http://blogs.cs.st-andrews.ac.uk/numonic                                *
 *                                                                         *
 * This file is part of numonic, a network and resource monitoring         *
 * package.                                                                *
 *                                                                         *
 * numonic is free software: you can redistribute it and/or                *
 * modify it under the terms of the GNU General Public License as          *
 * published by the Free Software Foundation, either version 3 of the      *
 * License, or (at your option) any later version.                         *
 *                                                                         *
 * numonic is distributed in the hope that it will be useful,              *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 * GNU General Public License for more details.                            *
 *                                                                         *
 * You should have received a copy of the GNU General Public License       *
 * along with numonic.  If not, see <http://www.gnu.org/licenses/>.        *
 *                                                                         *
 ***************************************************************************/

package uk.ac.standrews.cs.numonic.monitor.bandwidthAndLatency.test;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.junit.Test;

import uk.ac.standrews.cs.nds.madface.HostDescriptor;
import uk.ac.standrews.cs.nds.madface.JavaProcessDescriptor;
import uk.ac.standrews.cs.nds.madface.ProcessDescriptor;
import uk.ac.standrews.cs.nds.madface.URL;
import uk.ac.standrews.cs.numonic.madface.NumonicManager;
import uk.ac.standrews.cs.numonic.monitor.bandwidthAndLatency.BandwidthAndLatencyClient;
import uk.ac.standrews.cs.numonic.monitor.bandwidthAndLatency.BandwidthAndLatencyServer;

/**
 * Bandwidth and latency tests, not intended to be run automatically.
 *
 * @author Graham Kirby (graham.kirby@st-andrews.ac.uk)
 */
public class BandwidthAndLatencyManualTests {

    private Set<URL> getURLs() throws IOException {

        final Set<URL> urls = new HashSet<URL>();

        urls.add(new URL("http://www-systems.cs.st-andrews.ac.uk:8080/hudson/job/numonic/lastStableBuild/artifact/bin/numonic.jar"));
        urls.add(new URL("http://www-systems.cs.st-andrews.ac.uk:8080/hudson/job/numonic/lastSuccessfulBuild/artifact/lib/libsigar-amd64-linux.so"));
        urls.add(new URL("http://www-systems.cs.st-andrews.ac.uk:8080/hudson/job/numonic/lastSuccessfulBuild/artifact/lib/libsigar-universal-macosx.dylib"));
        urls.add(new URL("http://www-systems.cs.st-andrews.ac.uk:8080/hudson/job/numonic/lastSuccessfulBuild/artifact/lib/libsigar-universal64-macosx.dylib"));
        urls.add(new URL("http://www-systems.cs.st-andrews.ac.uk:8080/hudson/job/numonic/lastSuccessfulBuild/artifact/lib/libsigar-x86-linux.so"));
        urls.add(new URL("http://www-systems.cs.st-andrews.ac.uk:8080/hudson/job/numonic/lastSuccessfulBuild/artifact/lib/sigar.jar"));

        return urls;
    }

    /**
     * Tests bandwidth and latency measurement between local and remote machines.
     * @throws Exception if an error occurs
     */
    @Test
    public void localAndRemoteMachineMeasurement() throws Exception {

        final HostDescriptor host_descriptor = new HostDescriptor(true).applicationURLs(getURLs()).javaBinPath(NumonicManager.JAVA_BIN_PATH);
        final ProcessDescriptor java_process_descriptor1 = new JavaProcessDescriptor().classToBeInvoked(BandwidthAndLatencyServer.class);
        final Process p1 = host_descriptor.getProcessManager().runProcess(java_process_descriptor1);

        final String remote_machine_name = host_descriptor.getHost();

        final List<String> args = new ArrayList<String>();
        args.add(remote_machine_name);

        final ProcessDescriptor java_process_descriptor2 = new JavaProcessDescriptor().classToBeInvoked(BandwidthAndLatencyClient.class).args(args);
        final Process p2 = new HostDescriptor().getProcessManager().runProcess(java_process_descriptor2);

        p2.waitFor();
        p1.destroy();
    }

    /**
     * Tests bandwidth and latency measurement between remote machines.
     * @throws Exception if an error occurs
     */
    @Test
    public void twoRemoteMachinesMeasurement() throws Exception {

        System.out.println("Enter server details:");
        final HostDescriptor server = new HostDescriptor(true).applicationURLs(getURLs()).javaBinPath(NumonicManager.JAVA_BIN_PATH);

        System.out.println("Enter client details:");
        final HostDescriptor client = new HostDescriptor(true).applicationURLs(getURLs()).javaBinPath(NumonicManager.JAVA_BIN_PATH);

        final ProcessDescriptor java_process_descriptor1 = new JavaProcessDescriptor().classToBeInvoked(BandwidthAndLatencyServer.class);
        final Process p1 = client.getProcessManager().runProcess(java_process_descriptor1);

        final String server_machine_name = server.getHost();

        final List<String> args = new ArrayList<String>();
        args.add(server_machine_name);

        final ProcessDescriptor java_process_descriptor2 = new JavaProcessDescriptor().classToBeInvoked(BandwidthAndLatencyClient.class).args(args);
        final Process p2 = client.getProcessManager().runProcess(java_process_descriptor2);

        p2.waitFor();
        p1.destroy();
    }
}
