/***************************************************************************
 *                                                                         *
 * numonic Library                                                         *
 * Copyright (C) 2009-2011 Distributed Systems Architecture Research Group *
 * University of St Andrews, Scotland                                      *
 * http://blogs.cs.st-andrews.ac.uk/numonic                                    *
 *                                                                         *
 * This file is part of remote_management, a package providing             *
 * functionality for remotely managing a specified application.            *
 *                                                                         *
 * numonic is free software: you can redistribute it and/or                *
 * modify it under the terms of the GNU General Public License as          *
 * published by the Free Software Foundation, either version 3 of the      *
 * License, or (at your option) any later version.                         *
 *                                                                         *
 * numonic is distributed in the hope that it will be useful,              *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 * GNU General Public License for more details.                            *
 *                                                                         *
 * You should have received a copy of the GNU General Public License       *
 * along with numonic.  If not, see <http://www.gnu.org/licenses/>.        *
 *                                                                         *
 ***************************************************************************/
package uk.ac.standrews.cs.numonic.sort.ranker;

/**
 * The maximum values given to each resource across all of the machines being compared.
 * This data is used to normalise the values of resources when taking them into account in ranking machines.
 *
 * @author Angus Macdonald (angus AT cs.st-andrews.ac.uk)
 */
public class Bounds {

    private double cpu_max;
    private double mem_max;

    // -------------------------------------------------------------------------------------------------------

    /**
     * Initializes a record.
     *
     * @param cpu_max the maximum CPU value
     * @param mem_max the maximum memory value
     */
    public Bounds(final double cpu_max, final double mem_max) {

        this.cpu_max = cpu_max;
        this.mem_max = mem_max;
    }

    /**
     * Returns the maximum CPU value.
     * @return the maximum CPU value
     */
    public double getCpuMax() {

        return cpu_max;
    }

    /**
     * Returns the maximum memory value.
     * @return the maximum memory value
     */
    public double getMemMax() {

        return mem_max;
    }

    /**
     * Sets the maximum CPU value.
     * @param cpu_max the maximum CPU value
     */
    public void setCpuMax(final double cpu_max) {

        this.cpu_max = cpu_max;
    }

    /**
     * Sets the maximum memory value.
     * @param mem_max the maximum memory value
     */
    public void setMemMax(final double mem_max) {

        this.mem_max = mem_max;
    }
}
