/***************************************************************************
 *                                                                         *
 * numonic Library                                                         *
 * Copyright (C) 2009-2011 Distributed Systems Architecture Research Group *
 * University of St Andrews, Scotland                                      *
 * http://blogs.cs.st-andrews.ac.uk/numonic                                    *
 *                                                                         *
 * This file is part of remote_management, a package providing             *
 * functionality for remotely managing a specified application.            *
 *                                                                         *
 * numonic is free software: you can redistribute it and/or                *
 * modify it under the terms of the GNU General Public License as          *
 * published by the Free Software Foundation, either version 3 of the      *
 * License, or (at your option) any later version.                         *
 *                                                                         *
 * numonic is distributed in the hope that it will be useful,              *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 * GNU General Public License for more details.                            *
 *                                                                         *
 * You should have received a copy of the GNU General Public License       *
 * along with numonic.  If not, see <http://www.gnu.org/licenses/>.        *
 *                                                                         *
 ***************************************************************************/
package uk.ac.standrews.cs.numonic.util;

/**
 * Utility class defining time constants.
 *
 * @author Alan Dearle (al@cs.st-andrews.ac.uk)
 */
public final class Time {

    private Time() {

    }

    /**
     * The number of milliseconds in a second.
     */
    public static final int ONE_SECOND_IN_MS = 1000;

    /**
     * The number of milliseconds in a minute.
     */
    public static final int ONE_MINUTE_IN_MS = ONE_SECOND_IN_MS * 60;

    /**
     * The number of milliseconds in five minutes.
     */
    public static final int FIVE_MINUTES_IN_MS = ONE_MINUTE_IN_MS * 5;

    /**
     * The number of milliseconds in an hour.
     */
    public static final int ONE_HOUR_IN_MS = ONE_MINUTE_IN_MS * 60;

    /**
     * The number of milliseconds in eight hours.
     */
    public static final int EIGHT_HOURS_IN_MS = ONE_HOUR_IN_MS * 8;

    /**
     * The number of milliseconds in a day.
     */
    public static final int ONE_DAY_IN_MS = ONE_HOUR_IN_MS * 24;

    /**
     * The number of milliseconds in five days.
     */
    public static final long FIVE_DAYS_IN_MS = ONE_DAY_IN_MS * 5;
}
