/***************************************************************************
 *                                                                         *
 * nds Library                                                             *
 * Copyright (C) 2005-2010 Distributed Systems Architecture Research Group *
 * University of St Andrews, Scotland                                      *
 * http://www-systems.cs.st-andrews.ac.uk/                                 *
 *                                                                         *
 * This file is part of nds, a package of utility classes.                 *
 *                                                                         *
 * nds is free software: you can redistribute it and/or modify             *
 * it under the terms of the GNU General Public License as published by    *
 * the Free Software Foundation, either version 3 of the License, or       *
 * (at your option) any later version.                                     *
 *                                                                         *
 * nds is distributed in the hope that it will be useful,                  *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 * GNU General Public License for more details.                            *
 *                                                                         *
 * You should have received a copy of the GNU General Public License       *
 * along with nds.  If not, see <http://www.gnu.org/licenses/>.            *
 *                                                                         *
 ***************************************************************************/
package uk.ac.standrews.cs.nds.events;

import java.util.HashMap;

/**
 * Generic event implementation.
 *
 * @author Graham Kirby (graham.kirby@st-andrews.ac.uk)
 */
public class Event extends HashMap<String, Object> implements IEvent {

    private static final long serialVersionUID = 5760492894069559493L;
    private String type = "";

    /**
     * Creates an event with a given type.
     *
     * @param type the event type
     */
    public Event(final String type) {

        this.type = type;
    }

    @Override
    public String getType() {

        return type;
    }

    @Override
    public String toString() {

        return type + ": " + super.toString();
    }

    @Override
    public int hashCode() {

        final int prime = 31;
        int result = super.hashCode();
        result = prime * result + (type == null ? 0 : type.hashCode());
        return result;
    }

    @Override
    public boolean equals(final Object obj) {

        return obj instanceof Event && ((Event) obj).type.equals(type) && super.equals(obj);
    }
}
