/***************************************************************************
 *                                                                         *
 * nds Library                                                             *
 * Copyright (C) 2005-2010 Distributed Systems Architecture Research Group *
 * University of St Andrews, Scotland                                      *
 * http://www-systems.cs.st-andrews.ac.uk/                                 *
 *                                                                         *
 * This file is part of nds, a package of utility classes.                 *
 *                                                                         *
 * nds is free software: you can redistribute it and/or modify             *
 * it under the terms of the GNU General Public License as published by    *
 * the Free Software Foundation, either version 3 of the License, or       *
 * (at your option) any later version.                                     *
 *                                                                         *
 * nds is distributed in the hope that it will be useful,                  *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 * GNU General Public License for more details.                            *
 *                                                                         *
 * You should have received a copy of the GNU General Public License       *
 * along with nds.  If not, see <http://www.gnu.org/licenses/>.            *
 *                                                                         *
 ***************************************************************************/
package uk.ac.standrews.cs.nds.events.bus;

import java.util.ArrayList;
import java.util.Collection;

import uk.ac.standrews.cs.nds.events.IEvent;
import uk.ac.standrews.cs.nds.events.bus.interfaces.IEventBus;
import uk.ac.standrews.cs.nds.events.bus.interfaces.IEventConsumer;

/**
 * Event bus implementation.
 * 
 * @author Stuart Norcross (stuart@cs.st-andrews.ac.uk)
 * @author Graham Kirby (graham.kirby@st-andrews.ac.uk)
 */
public class EventBus implements IEventBus {

    private final Collection<IEventConsumer> registered_consumers;

    /**
     * Creates a new event bus.
     */
    public EventBus() {

        registered_consumers = new ArrayList<IEventConsumer>();
    }

    /**
     * Distributes the given event to all interested consumers.
     *
     * @param event the event to be distributed
     */
    @Override
    public void publishEvent(final IEvent event) {

        for (final IEventConsumer event_consumer : registered_consumers) {
            if (event_consumer.interested(event)) {
                event_consumer.receiveEvent(event);
            }
        }
    }

    /**
     * Registers a new consumer with the event bus.
     *
     * @param consumer the event consumer to be registered
     */
    @Override
    public void register(final IEventConsumer consumer) {

        registered_consumers.add(consumer);
    }

    /**
     * Removes a consumer from the event bus.
     *
     * @param consumer the event consumer to be removed
     */
    @Override
    public void unregister(final IEventConsumer consumer) {

        registered_consumers.remove(consumer);
    }
}
