package uk.ac.standrews.cs.nds.madface;

import java.io.OutputStream;
import java.util.concurrent.TimeUnit;

import uk.ac.standrews.cs.nds.madface.interfaces.IStreamProcessor;
import uk.ac.standrews.cs.nds.util.Duration;
import uk.ac.standrews.cs.nds.util.TimeoutExecutor;

public class ProcessDescriptor {

    private static final Duration DEFAULT_PROCESS_TIMEOUT = new Duration(10, TimeUnit.SECONDS);
    private volatile OutputStream output_stream;
    private volatile OutputStream error_stream;
    private volatile IStreamProcessor output_processor;
    private volatile IStreamProcessor error_processor;
    private volatile TimeoutExecutor timeout_executor;
    private volatile String command;
    private volatile String label;

    public OutputStream getOutputStream() {

        return output_stream == null ? System.out : output_stream;
    }

    public IStreamProcessor getOutputProcessor() {

        return output_processor;
    }

    public IStreamProcessor getErrorProcessor() {

        return error_processor;
    }

    public OutputStream getErrorStream() {

        return error_stream == null ? System.err : error_stream;
    }

    public synchronized TimeoutExecutor getExecutor() {

        if (timeout_executor == null) {
            timeout_executor = makeDefaultTimeoutExecutor();
        }
        return timeout_executor;
    }

    public void shutdown() {

        if (timeout_executor != null) {
            timeout_executor.shutdown();
        }
    }

    public ProcessDescriptor executor(final TimeoutExecutor timeout_executor) {

        this.timeout_executor = timeout_executor;
        return this;
    }

    public ProcessDescriptor command(final String command) {

        this.command = command;
        return this;
    }

    public ProcessDescriptor label(final String label) {

        this.label = label;
        return this;
    }

    public ProcessDescriptor outputStream(final OutputStream output_stream) {

        this.output_stream = output_stream;
        return this;
    }

    public ProcessDescriptor errorStream(final OutputStream error_stream) {

        this.error_stream = error_stream;
        return this;
    }

    public String getCommand() {

        return command;
    }

    public String getLabel() {

        return label == null ? command : label;
    }

    public ProcessDescriptor outputProcessor(final IStreamProcessor output_processor) {

        this.output_processor = output_processor;
        return this;
    }

    private TimeoutExecutor makeDefaultTimeoutExecutor() {

        return new TimeoutExecutor(1, DEFAULT_PROCESS_TIMEOUT, true, "default process executor");
    }
}
