/***************************************************************************
 *                                                                         *
 * nds Library                                                             *
 * Copyright (C) 2005-2011 Distributed Systems Architecture Research Group *
 * University of St Andrews, Scotland                                      *
 * http://www-systems.cs.st-andrews.ac.uk/                                 *
 *                                                                         *
 * This file is part of nds, a package of utility classes.                 *
 *                                                                         *
 * nds is free software: you can redistribute it and/or modify             *
 * it under the terms of the GNU General Public License as published by    *
 * the Free Software Foundation, either version 3 of the License, or       *
 * (at your option) any later version.                                     *
 *                                                                         *
 * nds is distributed in the hope that it will be useful,                  *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 * GNU General Public License for more details.                            *
 *                                                                         *
 * You should have received a copy of the GNU General Public License       *
 * along with nds.  If not, see <http://www.gnu.org/licenses/>.            *
 *                                                                         *
 ***************************************************************************/
package uk.ac.standrews.cs.nds.madface.scanners;

import java.util.concurrent.TimeUnit;

import uk.ac.standrews.cs.nds.madface.interfaces.IHostScanner;
import uk.ac.standrews.cs.nds.util.Duration;
import uk.ac.standrews.cs.nds.util.TimeoutExecutor;

/**
 * Common scanner functionality.
 *
 * @author Graham Kirby (graham.kirby@st-andrews.ac.uk)
 */
public abstract class Scanner implements IHostScanner {

    public static Duration DEFAULT_MIN_CYCLE_TIME = new Duration(2, TimeUnit.SECONDS); // The minimum time between successive scans.

    private Duration min_cycle_time = DEFAULT_MIN_CYCLE_TIME;

    private final TimeoutExecutor timeout_executor;

    protected boolean enabled = enabledByDefault();

    // -------------------------------------------------------------------------------------------------------

    /**
     * Initializes the scanner.
     */
    public Scanner() {

        timeout_executor = makeExecutor();
    }

    /**
     * Returns a new executor for the scanner.
     * @return a new executor for the scanner
     */
    // -------------------------------------------------------------------------------------------------------

    public abstract TimeoutExecutor makeExecutor();

    // -------------------------------------------------------------------------------------------------------

    @Override
    public Duration getMinCycleTime() {

        return min_cycle_time;
    }

    @Override
    public void setMinCycleTime(final Duration min_cycle_time) {

        this.min_cycle_time = min_cycle_time;
    }

    @Override
    public void setEnabled(final boolean enabled) {

        this.enabled = enabled;
    }

    @Override
    public TimeoutExecutor getTimeoutExecutor() {

        return timeout_executor;
    }

    @Override
    public void shutdown() {

        timeout_executor.shutdown();
    }
}
