/***************************************************************************
 *                                                                         *
 * nds Library                                                             *
 * Copyright (C) 2005-2011 Distributed Systems Architecture Research Group *
 * University of St Andrews, Scotland                                      *
 * http://www-systems.cs.st-andrews.ac.uk/                                 *
 *                                                                         *
 * This file is part of nds, a package of utility classes.                 *
 *                                                                         *
 * nds is free software: you can redistribute it and/or modify             *
 * it under the terms of the GNU General Public License as published by    *
 * the Free Software Foundation, either version 3 of the License, or       *
 * (at your option) any later version.                                     *
 *                                                                         *
 * nds is distributed in the hope that it will be useful,                  *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 * GNU General Public License for more details.                            *
 *                                                                         *
 * You should have received a copy of the GNU General Public License       *
 * along with nds.  If not, see <http://www.gnu.org/licenses/>.            *
 *                                                                         *
 ***************************************************************************/
package uk.ac.standrews.cs.nds.registry;

import java.util.Map;

import uk.ac.standrews.cs.nds.rpc.RPCException;

/**
 * Registry interface.
 *
 * @author Alan Dearle (al@cs.st-andrews.ac.uk)
 * @author Angus Macdonald (angus@cs.st-andrews.ac.uk)
 * @author Graham Kirby (graham.kirby@st-andrews.ac.uk)
 */
public interface IRegistry {

    /**
     * Adds a binding to the registry.
     *
     * @param key the registry key
     * @param port the port
     * @throws RPCException if an error occurs during the remote call
     * @throws AlreadyBoundException if there is already a binding for the given key
     */
    void bind(final String key, final int port) throws RPCException, AlreadyBoundException;

    /**
     * Creates a binding in the registry whether there is one there or not.
     *
     * @param key the registry key
     * @param port the port
     * @throws RPCException if an error occurs during the remote call
     */
    void rebind(final String key, final int port) throws RPCException;

    /**
     * Looks up a binding in the registry.
     *
     * @param key the registry key
     * @return the corresponding port
     * @throws RPCException if an error occurs during the remote call
     */
    int lookup(final String key) throws RPCException;

    /**
     * Removes a binding from the registry.
     *
     * @param key the registry key
     * @throws RPCException if an error occurs during the remote call
     * @throws NotBoundException if there is no binding for the given key
     */
    void unbind(final String key) throws RPCException, NotBoundException;

    /**
     * Returns the registry map.
     *
     * @return the registry map
     * @throws RPCException if an error occurs during the remote call
     */
    Map<String, Integer> getEntries() throws RPCException;

    void shutdown() throws RPCException, NotBoundException;
}
