/***************************************************************************
 *                                                                         *
 * nds Library                                                             *
 * Copyright (C) 2005-2011 Distributed Systems Architecture Research Group *
 * University of St Andrews, Scotland                                      *
 * http://www-systems.cs.st-andrews.ac.uk/                                 *
 *                                                                         *
 * This file is part of nds, a package of utility classes.                 *
 *                                                                         *
 * nds is free software: you can redistribute it and/or modify             *
 * it under the terms of the GNU General Public License as published by    *
 * the Free Software Foundation, either version 3 of the License, or       *
 * (at your option) any later version.                                     *
 *                                                                         *
 * nds is distributed in the hope that it will be useful,                  *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 * GNU General Public License for more details.                            *
 *                                                                         *
 * You should have received a copy of the GNU General Public License       *
 * along with nds.  If not, see <http://www.gnu.org/licenses/>.            *
 *                                                                         *
 ***************************************************************************/
package uk.ac.standrews.cs.nds.registry.stream;

import java.io.IOException;
import java.net.InetAddress;
import java.net.InetSocketAddress;

import uk.ac.standrews.cs.nds.registry.AbstractRegistryFactory;
import uk.ac.standrews.cs.nds.registry.AlreadyBoundException;
import uk.ac.standrews.cs.nds.registry.IRegistry;
import uk.ac.standrews.cs.nds.registry.Registry;
import uk.ac.standrews.cs.nds.registry.RegistryUnavailableException;
import uk.ac.standrews.cs.nds.rpc.RPCException;

/**
 * Provides access to the registry for a local or remote machine.
 *
 * @author Alan Dearle (al@cs.st-andrews.ac.uk)
 * @author Angus Macdonald (angus@cs.st-andrews.ac.uk)
 * @author Graham Kirby (graham.kirby@st-andrews.ac.uk)
 * @author Masih Hajiarabderkani (masih@cs.st-andrews.ac.uk)
 */
public final class RegistryFactory extends AbstractRegistryFactory {

    public static final RegistryFactory FACTORY = new RegistryFactory();

    /**
     * Returns the currently running registry for the a remote machine.
     *
     * @param host the remote address
     * @return the remote registry
     * @throws RegistryUnavailableException if the remote registry is not accessible
     */
    @Override
    public IRegistry getRegistry(final InetAddress host) throws RegistryUnavailableException {

        try {
            final InetSocketAddress address = new InetSocketAddress(host, RegistryServer.REGISTRY_PORT);
            final RegistryProxy registry_proxy = RegistryProxy.getProxy(address);

            registry_proxy.ping();
            return registry_proxy;
        }
        catch (final RPCException e) {
            throw new RegistryUnavailableException(e);
        }
    }

    // -------------------------------------------------------------------------------------------------------

    @Override
    public IRegistry startNewRegistry() throws IOException, RPCException, AlreadyBoundException, RegistryUnavailableException {

        final Registry registry = new Registry();
        new RegistryServer(registry).start();

        // Successfully started registry.
        return registry;
    }
}
