/***************************************************************************
 *                                                                         *
 * nds Library                                                             *
 * Copyright (C) 2005-2011 Distributed Systems Architecture Research Group *
 * University of St Andrews, Scotland                                      *
 * http://www-systems.cs.st-andrews.ac.uk/                                 *
 *                                                                         *
 * This file is part of nds, a package of utility classes.                 *
 *                                                                         *
 * nds is free software: you can redistribute it and/or modify             *
 * it under the terms of the GNU General Public License as published by    *
 * the Free Software Foundation, either version 3 of the License, or       *
 * (at your option) any later version.                                     *
 *                                                                         *
 * nds is distributed in the hope that it will be useful,                  *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 * GNU General Public License for more details.                            *
 *                                                                         *
 * You should have received a copy of the GNU General Public License       *
 * along with nds.  If not, see <http://www.gnu.org/licenses/>.            *
 *                                                                         *
 ***************************************************************************/
package uk.ac.standrews.cs.nds.registry.stream.test;

import static org.junit.Assert.assertEquals;

import java.io.StringReader;
import java.io.StringWriter;
import java.io.Writer;
import java.util.HashMap;
import java.util.Map;

import org.json.JSONException;
import org.json.JSONWriter;
import org.junit.Before;
import org.junit.Test;

import uk.ac.standrews.cs.nds.registry.stream.RegistryMarshaller;
import uk.ac.standrews.cs.nds.rpc.DeserializationException;
import uk.ac.standrews.cs.nds.rpc.stream.JSONReader;

/**
 * Tests registry marshalling code.
 *
 * @author Graham Kirby (graham.kirby@st-andrews.ac.uk)
 * @author Masih Hajiarabderkani (masih@cs.st-andrews.ac.uk)
 */
public class RegistryMarshallerTest {

    private RegistryMarshaller marshaller;
    private Map<String, Integer> map;

    /**
     * Sets up the test map.
     */
    @Before
    public void setup() {

        marshaller = new RegistryMarshaller();
        map = new HashMap<String, Integer>();

        final int val1 = 30001;
        final int val2 = 62100;
        final int val3 = 61100;
        final int val4 = 60100;

        map.put("CHORD", val1);
        map.put("eb5852fa10f0ba29881f8dae49c5178f8558f94d", val2);
        map.put("H2O_DATABASE", val2);
        map.put("H2O_SYSTEM_TABLE", val3);
        map.put("H2O_TABLE_MANAGER_INSTANCE", val4);
    }

    /**
     * Tests serialization and deserialization of a map.
     *
     * @throws DeserializationException if deserialization fails
     * @throws JSONException if json syntax error
     */
    @Test
    public void mapMarshalling() throws DeserializationException, JSONException {

        final Writer sw = new StringWriter();
        final JSONWriter writer = new JSONWriter(sw);

        marshaller.serializeMapStringInt(map, writer);
        final String s = sw.toString();
        System.out.println("Created String: " + s);

        final JSONReader reader = new JSONReader(new StringReader(s));
        reader.nextSymbol(); // start the parse

        final Map<String, Integer> deserialized = marshaller.deserializeMapStringInt(reader);

        assertEquals(map, deserialized);
    }
}
