/***************************************************************************
 *                                                                         *
 * nds Library                                                             *
 * Copyright (C) 2005-2011 Distributed Systems Architecture Research Group *
 * University of St Andrews, Scotland                                      *
 * http://www-systems.cs.st-andrews.ac.uk/                                 *
 *                                                                         *
 * This file is part of nds, a package of utility classes.                 *
 *                                                                         *
 * nds is free software: you can redistribute it and/or modify             *
 * it under the terms of the GNU General Public License as published by    *
 * the Free Software Foundation, either version 3 of the License, or       *
 * (at your option) any later version.                                     *
 *                                                                         *
 * nds is distributed in the hope that it will be useful,                  *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 * GNU General Public License for more details.                            *
 *                                                                         *
 * You should have received a copy of the GNU General Public License       *
 * along with nds.  If not, see <http://www.gnu.org/licenses/>.            *
 *                                                                         *
 ***************************************************************************/
package uk.ac.standrews.cs.nds.rpc;

import java.io.IOException;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.ServerSocket;

/**
 * Generic server-side RPC mechanism.
 *
 * @author Graham Kirby (graham.kirby@st-andrews.ac.uk)
 */
public abstract class AbstractApplicationServer {

    public static final String PING_NAME = "$checkServerConnection";

    /** The JSON key for call errors. */
    public static final String ERROR_KEY = "error";

    protected InetAddress local_address;
    protected int port;
    protected ServerSocket server_socket;
    protected String registry_key;

    /**
     * Returns the registry key to be used for the application.
     * @return the registry key to be used for the application
     */
    public abstract String getApplicationRegistryKey();

    public abstract void runServer(InetAddress address, int port) throws Exception;

    public abstract void runServer(String[] args) throws Exception;

    public abstract void stop() throws IOException;

    /**
     * Sets the local address of the server.
     * @param local_address the local address
     */
    public void setLocalAddress(final InetAddress local_address) {

        this.local_address = local_address;
    }

    /**
     * Returns this server's address.
     * @return this server's address.
     */
    public InetSocketAddress getAddress() {

        return new InetSocketAddress(local_address, port);
    }

    /**
     * Sets the port on which the server listens.
     * @param port the port
     */
    public void setPort(final int port) {

        this.port = port;
    }
}
