package uk.ac.standrews.cs.nds.rpc;

import java.io.BufferedReader;
import java.net.InetSocketAddress;
import java.net.Socket;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicInteger;

import uk.ac.standrews.cs.nds.util.Duration;

public abstract class AbstractConnection {

    protected final Socket socket;
    protected final InetSocketAddress address;
    protected BufferedReader input_stream;

    protected final int id;
    protected long last_released;

    private static final AtomicInteger next_id = new AtomicInteger(0);

    // -------------------------------------------------------------------------------------------------------------------------------

    private static final Duration GARBAGE_TIMEOUT = new Duration(20, TimeUnit.SECONDS);
    private static final long GARBAGE_TIMEOUT_IN_MILLIS = GARBAGE_TIMEOUT.getLength(TimeUnit.MILLISECONDS);

    // -------------------------------------------------------------------------------------------------------------------------------

    /**
     * Instantiates a new abstract connection.
     *
     * @param socket the socket
     */
    protected AbstractConnection(final Socket socket) {

        this.socket = socket;
        address = (InetSocketAddress) socket.getRemoteSocketAddress();
        id = next_id.getAndIncrement();
    }

    /**
     * Closes the input and output streams.
     */
    public abstract void tearDown();

    public long lastReleased() {

        return last_released;
    }

    public void recordReleaseTime() {

        last_released = System.currentTimeMillis();
    }

    /**
     * Gets the socket.
     *
     * @return the socket
     */
    public Socket getSocket() {

        return socket;
    }

    public InetSocketAddress getAddress() {

        return address;
    }

    public boolean isGarbage() {

        return System.currentTimeMillis() - last_released > GARBAGE_TIMEOUT_IN_MILLIS;
    }
}
