package uk.ac.standrews.cs.nds.rpc;

import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import uk.ac.standrews.cs.nds.util.Diagnostic;
import uk.ac.standrews.cs.nds.util.DiagnosticLevel;

public abstract class AbstractServerThread extends Thread {

    protected ServerSocket server_socket;
    protected ExecutorService thread_pool = Executors.newCachedThreadPool();

    public AbstractServerThread(final ServerSocket server_socket) {

        this.server_socket = server_socket;
    }

    protected abstract void shutDownHandlers();

    protected abstract void handleRequest(final Socket socket) throws IOException;

    public void startServer() {

        start();
    }

    public void stopServer() {

        try {
            server_socket.close();
        }
        catch (final IOException e) {
            Diagnostic.trace(DiagnosticLevel.RUN, "error closing server socket");
        }
        finally {

            shutDownHandlers();
            interrupt();
        }
    }

    @Override
    public void run() {

        while (!isInterrupted()) {
            try {
                final Socket socket = server_socket.accept();
                handleRequest(socket);
            }
            catch (final IOException e) {
                Diagnostic.trace(DiagnosticLevel.RUN, "error accepting connection: ", e.getMessage());
            }
        }
    }
}
