/***************************************************************************
 *                                                                         *
 * nds Library                                                             *
 * Copyright (C) 2005-2011 Distributed Systems Architecture Research Group *
 * University of St Andrews, Scotland                                      *
 * http://www-systems.cs.st-andrews.ac.uk/                                 *
 *                                                                         *
 * This file is part of nds, a package of utility classes.                 *
 *                                                                         *
 * nds is free software: you can redistribute it and/or modify             *
 * it under the terms of the GNU General Public License as published by    *
 * the Free Software Foundation, either version 3 of the License, or       *
 * (at your option) any later version.                                     *
 *                                                                         *
 * nds is distributed in the hope that it will be useful,                  *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 * GNU General Public License for more details.                            *
 *                                                                         *
 * You should have received a copy of the GNU General Public License       *
 * along with nds.  If not, see <http://www.gnu.org/licenses/>.            *
 *                                                                         *
 ***************************************************************************/
package uk.ac.standrews.cs.nds.rpc.nostream;

import java.io.IOException;
import java.net.InetSocketAddress;

import uk.ac.standrews.cs.nds.rpc.AbstractConnection;
import uk.ac.standrews.cs.nds.rpc.AbstractConnectionPool;

/**
 * Manages persistent socket connections. The connection pool is a map containing an entry for various server addresses (instances of InetSocketAddress). Each entry maps to a set of
 * free connections to the corresponding address. When a connection to a given address is required, it is taken from the connection pool if one exists. When a connection is released,
 * it is added to the connection pool for future reuse.
 * 
 * This class is thread-safe.
 *
 * @author Graham Kirby (graham.kirby@st-andrews.ac.uk)
 */
public final class NoStreamConnectionPool extends AbstractConnectionPool {

    @Override
    protected AbstractConnection makeConnection(final InetSocketAddress address) throws IOException {

        return new Connection(makeSocket(address));
    }

}
