/***************************************************************************
 *                                                                         *
 * nds Library                                                             *
 * Copyright (C) 2005-2011 Distributed Systems Architecture Research Group *
 * University of St Andrews, Scotland                                      *
 * http://www-systems.cs.st-andrews.ac.uk/                                 *
 *                                                                         *
 * This file is part of nds, a package of utility classes.                 *
 *                                                                         *
 * nds is free software: you can redistribute it and/or modify             *
 * it under the terms of the GNU General Public License as published by    *
 * the Free Software Foundation, either version 3 of the License, or       *
 * (at your option) any later version.                                     *
 *                                                                         *
 * nds is distributed in the hope that it will be useful,                  *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 * GNU General Public License for more details.                            *
 *                                                                         *
 * You should have received a copy of the GNU General Public License       *
 * along with nds.  If not, see <http://www.gnu.org/licenses/>.            *
 *                                                                         *
 ***************************************************************************/
package uk.ac.standrews.cs.nds.rpc.nostream.json;

import org.json.JSONException;

/**
 * Union class that represents any JSON value (object, array, int, double, long, boolean or string.
 * This compensates for the lack of a JSON superclass.
 *
 * @author Graham Kirby (graham.kirby@st-andrews.ac.uk)
 */
public abstract class JSONValue {

    protected Object primitive_value = null;

    // -------------------------------------------------------------------------------------------------------

    protected JSONValue() {

    }

    protected JSONValue(final Object primitive_value) {

        this.primitive_value = primitive_value;
    }

    // -------------------------------------------------------------------------------------------------------

    /**
     * Returns the represented value.
     *
     * @return the represented value
     */
    public abstract Object getValue();

    protected abstract void putSelfIntoObject(final String key, final JSONObject parent);

    protected abstract void putSelfIntoArray(final int index, final JSONArray parent);

    // -------------------------------------------------------------------------------------------------------

    @Override
    public String toString() {

        return primitive_value.toString();
    }

    // -------------------------------------------------------------------------------------------------------

    /**
     * Creates a value from an instance of {@link JSONObject}, {@link JSONArray}, {@link Boolean}, {@link Double}, {@link Integer}, {@link String} or {@link Long}.
     *
     * @param object the instance
     * @return the value
     * @throws JSONException if the instance is not of a legal class
     */
    public static JSONValue makeValue(final Object object) throws JSONException {

        if (object == null || object == JSONObject.NULL || object == org.json.JSONObject.NULL) { return JSONObject.NULL; }
        if (object instanceof org.json.JSONObject) { return new JSONObject((org.json.JSONObject) object); }
        if (object instanceof org.json.JSONArray) { return new JSONArray((org.json.JSONArray) object); }
        if (object instanceof Boolean) { return new JSONBoolean((Boolean) object); }
        if (object instanceof Double) { return new JSONDouble((Double) object); }
        if (object instanceof Integer) { return new JSONInteger((Integer) object); }
        if (object instanceof String) { return new JSONString((String) object); }
        if (object instanceof Long) { return new JSONLong((Long) object); }

        throw new JSONException("unknown JSON type: " + object.getClass().getName());
    }
}
