/***************************************************************************
 *                                                                         *
 * nds Library                                                             *
 * Copyright (C) 2005-2011 Distributed Systems Architecture Research Group *
 * University of St Andrews, Scotland                                      *
 * http://www-systems.cs.st-andrews.ac.uk/                                 *
 *                                                                         *
 * This file is part of nds, a package of utility classes.                 *
 *                                                                         *
 * nds is free software: you can redistribute it and/or modify             *
 * it under the terms of the GNU General Public License as published by    *
 * the Free Software Foundation, either version 3 of the License, or       *
 * (at your option) any later version.                                     *
 *                                                                         *
 * nds is distributed in the hope that it will be useful,                  *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 * GNU General Public License for more details.                            *
 *                                                                         *
 * You should have received a copy of the GNU General Public License       *
 * along with nds.  If not, see <http://www.gnu.org/licenses/>.            *
 *                                                                         *
 ***************************************************************************/
package uk.ac.standrews.cs.nds.rpc.security;

import java.security.PublicKey;

import uk.ac.standrews.cs.nds.rpc.security.exception.PublicKeyNotFoundLocalException;

/**
 * The Interface IPublicKeyManager.
 *
 * @author Masih Hajiarabderkani (masih@cs.st-andrews.ac.uk)
 */
public interface IPublicKeyManager {

    /**
     * Gets the Public Key of a remote node from the locally stored information.
     *
     * @param signer_id the signer identifier
     * @return the remote node's Public Key
     * @throws PublicKeyNotFoundLocalException the public key not found local exception
     */
    PublicKey getPublicKey(Object signer_id) throws PublicKeyNotFoundLocalException;

    /**
     * Store public key.
     *
     * @param signer_id the signer_id
     * @param public_key the public_key
     * @return the public keyp
     */
    PublicKey storePublicKey(Object signer_id, PublicKey public_key);

}
