/***************************************************************************
 *                                                                         *
 * nds Library                                                             *
 * Copyright (C) 2005-2011 Distributed Systems Architecture Research Group *
 * University of St Andrews, Scotland                                      *
 * http://www-systems.cs.st-andrews.ac.uk/                                 *
 *                                                                         *
 * This file is part of nds, a package of utility classes.                 *
 *                                                                         *
 * nds is free software: you can redistribute it and/or modify             *
 * it under the terms of the GNU General Public License as published by    *
 * the Free Software Foundation, either version 3 of the License, or       *
 * (at your option) any later version.                                     *
 *                                                                         *
 * nds is distributed in the hope that it will be useful,                  *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 * GNU General Public License for more details.                            *
 *                                                                         *
 * You should have received a copy of the GNU General Public License       *
 * along with nds.  If not, see <http://www.gnu.org/licenses/>.            *
 *                                                                         *
 ***************************************************************************/
package uk.ac.standrews.cs.nds.rpc.security;

import java.security.InvalidKeyException;
import java.security.SignatureException;

/**
 * The Interface ISigner.
 *
 * @author Masih Hajiarabderkani (masih@cs.st-andrews.ac.uk)
 */
public interface ISigner {

    /**
     * Gets the id.
     *
     * @return the id
     */
    Object getId();

    /**
     * Inits the sign.
     *
     * @throws InvalidKeyException the invalid key exception
     */
    void initSign() throws InvalidKeyException;

    /**
     * Update.
     *
     * @param source the source
     * @param offset the offset
     * @param length the length
     * @throws SignatureException the signature exception
     */
    void update(byte[] source, int offset, int length) throws SignatureException;

    /**
     * Sign.
     *
     * @return the byte[]
     * @throws SignatureException the signature exception
     */
    byte[] sign() throws SignatureException;

}
