/***************************************************************************
 *                                                                         *
 * nds Library                                                             *
 * Copyright (C) 2005-2011 Distributed Systems Architecture Research Group *
 * University of St Andrews, Scotland                                      *
 * http://www-systems.cs.st-andrews.ac.uk/                                 *
 *                                                                         *
 * This file is part of nds, a package of utility classes.                 *
 *                                                                         *
 * nds is free software: you can redistribute it and/or modify             *
 * it under the terms of the GNU General Public License as published by    *
 * the Free Software Foundation, either version 3 of the License, or       *
 * (at your option) any later version.                                     *
 *                                                                         *
 * nds is distributed in the hope that it will be useful,                  *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 * GNU General Public License for more details.                            *
 *                                                                         *
 * You should have received a copy of the GNU General Public License       *
 * along with nds.  If not, see <http://www.gnu.org/licenses/>.            *
 *                                                                         *
 ***************************************************************************/
package uk.ac.standrews.cs.nds.rpc.stream;

import java.io.IOException;
import java.net.Socket;

import org.json.JSONWriter;

import uk.ac.standrews.cs.nds.rpc.AbstractConnection;
import uk.ac.standrews.cs.nds.util.Diagnostic;
import uk.ac.standrews.cs.nds.util.DiagnosticLevel;

/**
 *
 * @author Masih Hajiarabderkani (masih@cs.st-andrews.ac.uk)
 */
public abstract class AbstractStreamConnection extends AbstractConnection {

    AbstractStreamConnection(final Socket socket) {

        super(socket);
    }

    // -------------------------------------------------------------------------------------------------------------------------------

    /**
    * Completes write request. Flushes the buffer, resets the write buffer and makes a new {@link JSONWriter}.
    *
    * @throws IOException Signals that an I/O exception has occurred.
    */
    public void completeWriteRequest() throws IOException {

        resetWriteBuffer();
        newJSONwriter();
    }

    @Override
    public void tearDown() {

        if (socket != null) {
            try {
                socket.close();
            }
            catch (final IOException e) {
                Diagnostic.trace(DiagnosticLevel.RUN, "error closing socket");
            }
        }
    }

    // -------------------------------------------------------------------------------------------------------

    @Override
    public String toString() {

        return "streampair:" + id;
    }

    /**
     * Creates a new JSON writer. Need to do this on each call since JSONWriter can only cope with one top level object.
     *
     * @throws IOException Signals that an I/O exception has occurred.
     */
    public abstract void newJSONwriter() throws IOException;

    /**
     * Gets the JSON writer.
     *
     * @return the JSON writer
     */
    public abstract JSONWriter getJSONwriter();

    /**
     * Gets the JSON reader.
     *
     * @return the JSON reader. Null, if the verification is enabled and the verification has failed.
     */
    public abstract JSONReader getJSONreader();

    /**
     * Resets write buffer.
     *
     * @throws IOException Signals that an I/O exception has occurred.
     */
    public abstract void resetWriteBuffer() throws IOException;
}
