/***************************************************************************
 *                                                                         *
 * nds Library                                                             *
 * Copyright (C) 2005-2011 Distributed Systems Architecture Research Group *
 * University of St Andrews, Scotland                                      *
 * http://www-systems.cs.st-andrews.ac.uk/                                 *
 *                                                                         *
 * This file is part of nds, a package of utility classes.                 *
 *                                                                         *
 * nds is free software: you can redistribute it and/or modify             *
 * it under the terms of the GNU General Public License as published by    *
 * the Free Software Foundation, either version 3 of the License, or       *
 * (at your option) any later version.                                     *
 *                                                                         *
 * nds is distributed in the hope that it will be useful,                  *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 * GNU General Public License for more details.                            *
 *                                                                         *
 * You should have received a copy of the GNU General Public License       *
 * along with nds.  If not, see <http://www.gnu.org/licenses/>.            *
 *                                                                         *
 ***************************************************************************/
package uk.ac.standrews.cs.nds.rpc.stream;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.Socket;

import org.json.JSONWriter;

import uk.ac.standrews.cs.nds.util.Diagnostic;
import uk.ac.standrews.cs.nds.util.DiagnosticLevel;

/**
 * Utility class binding together input and output streams for a given socket.
 *
 * @author Graham Kirby (graham.kirby@st-andrews.ac.uk)
 * @author Masih Hajiarabderkani (masih@cs.st-andrews.ac.uk)
 */
public class Connection extends AbstractStreamConnection {

    private final JSONReader json_reader;
    private final AbortableBufferedWriter buffer;

    protected JSONWriter json_writer;

    // -------------------------------------------------------------------------------------------------------

    /**
     * Creates a stream pair for a given socket.
     *
     * @param socket the socket
     * @throws IOException if the streams cannot be created
     */
    public Connection(final Socket socket) throws IOException {

        super(socket);

        input_stream = new BufferedReader(new InputStreamReader(socket.getInputStream()));
        json_reader = new JSONReader(input_stream, this);

        buffer = new AbortableBufferedWriter(socket.getOutputStream());
        newJSONwriter();
    }

    // -------------------------------------------------------------------------------------------------------
    // IConnection overridden methods.

    @Override
    public void newJSONwriter() throws IOException {

        json_writer = new JSONWriter(buffer);
    }

    @Override
    public JSONWriter getJSONwriter() {

        return json_writer;
    }

    @Override
    public JSONReader getJSONreader() {

        return json_reader;
    }

    @Override
    public void resetWriteBuffer() throws IOException {

        buffer.reset();
    }

    // -------------------------------------------------------------------------------------------------------
    // AbstractStreamConnection overridden methods.

    @Override
    public void completeWriteRequest() throws IOException {

        buffer.flush();
        super.completeWriteRequest();
    }

    @Override
    public void tearDown() {

        try {
            input_stream.close();
        }
        catch (final IOException e) {
            Diagnostic.trace(DiagnosticLevel.RUN, "error closing input stream");
        }
        finally {
            super.tearDown();
        }
    }
}
