/***************************************************************************
 * * nds Library * Copyright (C) 2005-2011 Distributed Systems Architecture Research Group * University of St Andrews, Scotland *
 * http://www-systems.cs.st-andrews.ac.uk/ * * This file is part of nds, a package of utility classes. * * nds is free software: you can
 * redistribute it and/or modify * it under the terms of the GNU General Public License as published by * the Free Software Foundation,
 * either version 3 of the License, or * (at your option) any later version. * * nds is distributed in the hope that it will be useful, *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the * GNU
 * General Public License for more details. * * You should have received a copy of the GNU General Public License * along with nds. If not,
 * see <http://www.gnu.org/licenses/>. * *
 ***************************************************************************/
package uk.ac.standrews.cs.nds.rpc.stream;

import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.HashSet;
import java.util.Set;

import uk.ac.standrews.cs.nds.rpc.AbstractServerThread;

/**
 * Server Thread to handle incoming requests.
 *
 * @author Graham Kirby (graham.kirby@st-andrews.ac.uk)
 * @author Alan Dearle  (al@st-andrews.ac.uk)
 */
class ServerThread extends AbstractServerThread {

    private final ApplicationServer application_server;
    private final HashSet<RequestHandler> extant_handlers = new HashSet<RequestHandler>();

    public ServerThread(final ApplicationServer application_server, final ServerSocket server_socket) {

        super(server_socket);
        this.application_server = application_server;
    }

    @SuppressWarnings("unchecked")
    @Override
    protected void shutDownHandlers() {

        // Shut down all the extant request handlers.
        // The set is cloned since the handler removes itself from the set on shutdown.
        for (final RequestHandler handler : (Set<RequestHandler>) extant_handlers.clone()) {
            handler.shutDown();
        }
    }

    @Override
    protected void handleRequest(final Socket socket) throws IOException {

        final Runnable makeRequest = application_server.makeRequest(socket, extant_handlers);
        thread_pool.execute(makeRequest);
    }
}
