/***************************************************************************
 *                                                                         *
 * nds Library                                                             *
 * Copyright (C) 2005-2011 Distributed Systems Architecture Research Group *
 * University of St Andrews, Scotland                                      *
 * http://www-systems.cs.st-andrews.ac.uk/                                 *
 *                                                                         *
 * This file is part of nds, a package of utility classes.                 *
 *                                                                         *
 * nds is free software: you can redistribute it and/or modify             *
 * it under the terms of the GNU General Public License as published by    *
 * the Free Software Foundation, either version 3 of the License, or       *
 * (at your option) any later version.                                     *
 *                                                                         *
 * nds is distributed in the hope that it will be useful,                  *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 * GNU General Public License for more details.                            *
 *                                                                         *
 * You should have received a copy of the GNU General Public License       *
 * along with nds.  If not, see <http://www.gnu.org/licenses/>.            *
 *                                                                         *
 ***************************************************************************/
package uk.ac.standrews.cs.nds.rpc.stream;

import java.io.IOException;
import java.net.InetSocketAddress;
import java.net.Socket;
import java.util.concurrent.TimeUnit;

import uk.ac.standrews.cs.nds.rpc.AbstractConnection;
import uk.ac.standrews.cs.nds.rpc.AbstractConnectionPool;
import uk.ac.standrews.cs.nds.util.Diagnostic;
import uk.ac.standrews.cs.nds.util.DiagnosticLevel;
import uk.ac.standrews.cs.nds.util.Duration;

/**
 * Manages persistent signable socket connections.
 * 
 * @author Graham Kirby (graham.kirby@st-andrews.ac.uk)
 * @author Masih Hajiarabderkani (masih@cs.st-andrews.ac.uk)
 */
public final class SignableConnectionPool extends AbstractConnectionPool {

    private static final Duration SOCKET_CREATION_TIMEOUT = new Duration(2, TimeUnit.SECONDS);
    private static final Duration SOCKET_READ_TIMEOUT = new Duration(20, TimeUnit.SECONDS);

    @Override
    protected AbstractConnection makeConnection(final InetSocketAddress address) throws IOException {

        return new SignableConnection(makeSocket(address));
    }

    @Override
    protected void tidyConnection(final AbstractConnection connection) {

        final SignableConnection signable_connection = (SignableConnection) connection;

        signable_connection.disableSigning();
        signable_connection.disableVerification();
        try {
            signable_connection.resetWriteBuffer();
        }
        catch (final IOException e) {
            Diagnostic.trace(DiagnosticLevel.RUN, "Couldn't reset write buffer");
        }
    }

    @Override
    public Socket makeSocket(final InetSocketAddress address) throws IOException {

        final Socket socket = new Socket();
        socket.connect(address, (int) SOCKET_CREATION_TIMEOUT.getLength(TimeUnit.MILLISECONDS));
        socket.setSoTimeout((int) SOCKET_READ_TIMEOUT.getLength(TimeUnit.MILLISECONDS));
        return socket;
    }
}
