/***************************************************************************
 *                                                                         *
 * stachord Library                                                        *
 * Copyright (C) 2004-2010 Distributed Systems Architecture Research Group *
 * University of St Andrews, Scotland
 * http://www-systems.cs.st-andrews.ac.uk/                                 *
 *                                                                         *
 * This file is part of stachord, an independent implementation of         *
 * the Chord protocol (http://pdos.csail.mit.edu/chord/).                  *
 *                                                                         *
 * stachord is free software: you can redistribute it and/or modify        *
 * it under the terms of the GNU General Public License as published by    *
 * the Free Software Foundation, either version 3 of the License, or       *
 * (at your option) any later version.                                     *
 *                                                                         *
 * stachord is distributed in the hope that it will be useful,             *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 * GNU General Public License for more details.                            *
 *                                                                         *
 * You should have received a copy of the GNU General Public License       *
 * along with stachord.  If not, see <http://www.gnu.org/licenses/>.       *
 *                                                                         *
 ***************************************************************************/

package uk.ac.standrews.cs.nds.rpc.test.app;

import uk.ac.standrews.cs.nds.rpc.RPCException;

/**
 * Interface for test service.
 *
 * @author Graham Kirby (graham.kirby@st-andrews.ac.uk)
 */
public interface ITest {

    /**
     * Tests whether the x fields of all the objects in the array are equal.
     * @param t an array
     * @return true if the array and its elements are not null and all the x fields are equal
     * @throws RPCException if the call fails
     */
    boolean allElementsHaveSameX(TestParameter1[] t) throws RPCException;

    /**
     * Tests whether the arrays referenced by the parameters are equal.
     * @param t1 an object
     * @param t2 an object
     * @return true if the objects and arrays are not null and the arrays are equal
     * @throws RPCException if the call fails
     */
    boolean arraysEqual(TestParameter2 t1, TestParameter2 t2) throws RPCException;

    /**
     * Does nothing.
     * @throws RPCException if the call fails
     */
    void check() throws RPCException;

    /**
     * Checks whether the parameter is 3.
     * @param i an integer
     * @return true if i is 3
     * @throws RPCException if the call fails
     */
    boolean equalToThree(int i) throws RPCException;

    /**
     * Tests whether the parameter is the empty string.
     * @param s a string
     * @return true if s is the empty string
     * @throws RPCException if the call fails
     */
    boolean isEmptyString(String s) throws RPCException;

    /**
      * Returns the negation of the parameter.
      * @param b a boolean
      * @return the negation of b
      * @throws RPCException if the call fails
      */
    boolean not(boolean b) throws RPCException;

    boolean passByReference(ITest other, int i) throws RPCException;

    /**
     * Returns an array containing two objects with x values of 5 and 7.
     * @return the array
     * @throws RPCException if the call fails
     */
    TestParameter1[] returnsArrayLength2With5And7() throws RPCException;

    /**
     * Returns the empty string.
     * @return the empty string
     * @throws RPCException if the call fails
     */
    String returnsEmptyString() throws RPCException;

    /**
     * Returns null.
     * @return the null value
     * @throws RPCException if the call fails
     */
    TestParameter1 returnsNull() throws RPCException;

    String returnsParam(String s) throws RPCException;

    /**
     * Returns an object with an x field value of 3.
     * @return the object
     * @throws RPCException if the call fails
     */
    TestParameter1 returnsWithX3() throws RPCException;

    /**
     * Throws an application-related exception.
     * @throws RPCException if the call fails
     * @throws TestException an application-related exception
     */
    void throwsApplicationException() throws RPCException, TestException;

    /**
     * Throws a null-pointer exception.
     * @throws RPCException if the call fails
     */
    void throwsNullPointerException() throws RPCException;

    /**
     * Tests whether the x field of the parameter is 3.
     * @param t an object
     * @return true if t is not null and the x field is 3
     * @throws RPCException if the call fails
     */
    boolean xIsThree(TestParameter1 t) throws RPCException;

    /**
     * Tests whether the x field of the first parameter is equal to the y field of the t3 field of the second parameter.
     * @param t1 an object
     * @param t2 an object
     * @return true if all objects are not null and the specified fields are equal
     * @throws RPCException if the call fails
     */
    boolean xSameAsY(TestParameter1 t1, TestParameter2 t2) throws RPCException;
}
