/***************************************************************************
 *                                                                         *
 * nds Library                                                             *
 * Copyright (C) 2005-2011 Distributed Systems Architecture Research Group *
 * University of St Andrews, Scotland                                      *
 * http://www-systems.cs.st-andrews.ac.uk/                                 *
 *                                                                         *
 * This file is part of nds, a package of utility classes.                 *
 *                                                                         *
 * nds is free software: you can redistribute it and/or modify             *
 * it under the terms of the GNU General Public License as published by    *
 * the Free Software Foundation, either version 3 of the License, or       *
 * (at your option) any later version.                                     *
 *                                                                         *
 * nds is distributed in the hope that it will be useful,                  *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 * GNU General Public License for more details.                            *
 *                                                                         *
 * You should have received a copy of the GNU General Public License       *
 * along with nds.  If not, see <http://www.gnu.org/licenses/>.            *
 *                                                                         *
 ***************************************************************************/
package uk.ac.standrews.cs.nds.rpc.test.app;

import uk.ac.standrews.cs.nds.rpc.RPCException;

/**
 * Implementation of test service.
 *
 * @author Graham Kirby (graham.kirby@st-andrews.ac.uk)
 */
public class TestImpl implements ITest {

    private static final int THREE = 3;
    private static final int FIVE = 5;
    private static final int SEVEN = 7;

    @Override
    public void check() throws RPCException {

    }

    @Override
    public boolean equalToThree(final int i) throws RPCException {

        return i == THREE;
    }

    @Override
    public boolean not(final boolean b) throws RPCException {

        return !b;
    }

    @Override
    public boolean xIsThree(final TestParameter1 t) throws RPCException {

        return t != null && t.getX() == THREE;
    }

    @Override
    public boolean isEmptyString(final String s) throws RPCException {

        return s.equals("");
    }

    @Override
    public boolean xSameAsY(final TestParameter1 t1, final TestParameter2 t2) throws RPCException {

        return t1 != null && t2 != null && t2.getT3() != null && t1.getX() == t2.getT3().getY();
    }

    @Override
    public boolean arraysEqual(final TestParameter2 t1, final TestParameter2 t2) throws RPCException {

        if (t1 == null || t2 == null) { return false; }
        if (t1.getI() == null && t2.getI() == null) { return true; }
        if (t1.getI() == null && t2.getI() != null) { return false; }
        if (t1.getI() != null && t2.getI() == null) { return false; }

        if (t1.getI().length != t2.getI().length) { return false; }

        for (int i = 0; i < t1.getI().length; i++) {
            if (t1.getI()[i] != t2.getI()[i]) { return false; }
        }
        return true;
    }

    @Override
    public boolean allElementsHaveSameX(final TestParameter1[] t) throws RPCException {

        if (t == null) { return false; }
        if (t.length == 0) { return true; }

        if (t[0] == null) { return false; }
        final int first = t[0].getX();
        for (int i = 1; i < t.length; i++) {
            if (t[i] == null || t[i].getX() != first) { return false; }
        }
        return true;
    }

    @Override
    public TestParameter1 returnsWithX3() throws RPCException {

        return new TestParameter1(THREE);
    }

    @Override
    public String returnsParam(final String s) throws RPCException {

        return s;
    }

    @Override
    public TestParameter1[] returnsArrayLength2With5And7() throws RPCException {

        return new TestParameter1[]{new TestParameter1(FIVE), new TestParameter1(SEVEN)};
    }

    @Override
    public void throwsApplicationException() throws RPCException, TestException {

        throw new TestException("test message");
    }

    @Override
    public void throwsNullPointerException() throws RPCException {

        throw new NullPointerException();
    }

    @Override
    public TestParameter1 returnsNull() throws RPCException {

        return null;
    }

    @Override
    public boolean passByReference(final ITest other, final int i) throws RPCException {

        return other.equalToThree(i);
    }

    @Override
    public String returnsEmptyString() throws RPCException {

        return "";
    }
}
