/***************************************************************************
 *                                                                         *
 * nds Library                                                             *
 * Copyright (C) 2005-2011 Distributed Systems Architecture Research Group *
 * University of St Andrews, Scotland                                      *
 * http://www-systems.cs.st-andrews.ac.uk/                                 *
 *                                                                         *
 * This file is part of nds, a package of utility classes.                 *
 *                                                                         *
 * nds is free software: you can redistribute it and/or modify             *
 * it under the terms of the GNU General Public License as published by    *
 * the Free Software Foundation, either version 3 of the License, or       *
 * (at your option) any later version.                                     *
 *                                                                         *
 * nds is distributed in the hope that it will be useful,                  *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 * GNU General Public License for more details.                            *
 *                                                                         *
 * You should have received a copy of the GNU General Public License       *
 * along with nds.  If not, see <http://www.gnu.org/licenses/>.            *
 *                                                                         *
 ***************************************************************************/
package uk.ac.standrews.cs.nds.rpc.test.app.nostream;

import java.net.InetSocketAddress;

import uk.ac.standrews.cs.nds.rpc.DeserializationException;
import uk.ac.standrews.cs.nds.rpc.RPCException;
import uk.ac.standrews.cs.nds.rpc.nostream.Marshaller;
import uk.ac.standrews.cs.nds.rpc.nostream.NoStreamProxy;
import uk.ac.standrews.cs.nds.rpc.nostream.json.JSONArray;
import uk.ac.standrews.cs.nds.rpc.nostream.json.JSONObject;
import uk.ac.standrews.cs.nds.rpc.nostream.json.JSONString;
import uk.ac.standrews.cs.nds.rpc.test.app.ITest;
import uk.ac.standrews.cs.nds.rpc.test.app.TestException;
import uk.ac.standrews.cs.nds.rpc.test.app.TestParameter1;
import uk.ac.standrews.cs.nds.rpc.test.app.TestParameter2;

/**
 * Proxy for test service.
 *
 * @author Graham Kirby (graham.kirby@st-andrews.ac.uk)
 */
public class TestProxy extends NoStreamProxy implements ITest {

    private final TestMarshaller marshaller;

    // -------------------------------------------------------------------------------------------------------

    /**
     * Creates a proxy for a given server address.
     * @param node_address the address
     */
    public TestProxy(final InetSocketAddress node_address) {

        super(node_address);
        marshaller = new TestMarshaller();
    }

    // -------------------------------------------------------------------------------------------------------

    @Override
    public Marshaller getMarshaller() {

        return marshaller;
    }

    // -------------------------------------------------------------------------------------------------------

    @Override
    public void check() throws RPCException {

        try {
            makeCall("check");
        }
        catch (final Exception e) {
            dealWithException(e);
        }
    }

    @Override
    public boolean equalToThree(final int i) throws RPCException {

        try {
            final JSONArray args = new JSONArray();
            args.put(i);
            return (Boolean) makeCall("equalToThree", args).getValue();
        }
        catch (final Exception e) {
            dealWithException(e);
            return false;
        }
    }

    @Override
    public boolean not(final boolean b) throws RPCException {

        try {
            final JSONArray args = new JSONArray();
            args.put(b);
            return (Boolean) makeCall("not", args).getValue();
        }
        catch (final Exception e) {
            dealWithException(e);
            return false;
        }
    }

    @Override
    public boolean xIsThree(final TestParameter1 t) throws RPCException {

        try {
            final JSONArray args = new JSONArray();
            args.put(marshaller.serializeTestParameter1(t));
            return (Boolean) makeCall("xIsThree", args).getValue();
        }
        catch (final Exception e) {
            dealWithException(e);
            return false;
        }
    }

    @Override
    public boolean xSameAsY(final TestParameter1 t1, final TestParameter2 t2) throws RPCException {

        try {
            final JSONArray args = new JSONArray();

            args.put(marshaller.serializeTestParameter1(t1));
            args.put(marshaller.serializeTestParameter2(t2));

            return (Boolean) makeCall("xSameAsY", args).getValue();
        }
        catch (final Exception e) {
            dealWithException(e);
            return false;
        }
    }

    @Override
    public boolean arraysEqual(final TestParameter2 t1, final TestParameter2 t2) throws RPCException {

        try {
            final JSONArray args = new JSONArray();

            args.put(marshaller.serializeTestParameter2(t1));
            args.put(marshaller.serializeTestParameter2(t2));

            return (Boolean) makeCall("arraysEqual", args).getValue();
        }
        catch (final Exception e) {
            dealWithException(e);
            return false;
        }
    }

    @Override
    public boolean allElementsHaveSameX(final TestParameter1[] t) throws RPCException {

        try {
            final JSONArray args = new JSONArray();
            args.put(marshaller.serializeTestParameter1Array(t));
            return (Boolean) makeCall("allElementsHaveSameX", args).getValue();
        }
        catch (final Exception e) {
            dealWithException(e);
            return false;
        }
    }

    @Override
    public TestParameter1 returnsWithX3() throws RPCException {

        try {
            return marshaller.deserializeTestParameter1((JSONObject) makeCall("returnsWithX3"));
        }
        catch (final DeserializationException e) {
            throw new RPCException(e);
        }
        catch (final Exception e) {
            dealWithException(e);
            return null;
        }
    }

    @Override
    public String returnsParam(final String s) throws RPCException {

        try {
            final JSONArray args = new JSONArray();
            args.put(s);
            return (String) ((JSONString) makeCall("returnsParam", args)).getValue();
        }
        catch (final DeserializationException e) {
            throw new RPCException(e);
        }
        catch (final Exception e) {
            dealWithException(e);
            return null;
        }
    }

    @Override
    public TestParameter1[] returnsArrayLength2With5And7() throws RPCException {

        try {
            return marshaller.deserializeTestParameter1Array((JSONArray) makeCall("returnsArrayLength2With5And7"));
        }
        catch (final DeserializationException e) {
            throw new RPCException(e);
        }
        catch (final Exception e) {
            dealWithException(e);
            return null;
        }
    }

    @Override
    public void throwsApplicationException() throws RPCException, TestException {

        try {
            makeCall("throwsApplicationException");
        }
        catch (final TestException e) {
            throw e;
        }
        catch (final Exception e) {
            dealWithException(e);
        }
    }

    @Override
    public void throwsNullPointerException() throws RPCException {

        try {
            makeCall("throwsNullPointerException");
        }
        catch (final Exception e) {
            dealWithException(e);
        }
    }

    @Override
    public TestParameter1 returnsNull() throws RPCException {

        try {
            return marshaller.deserializeTestParameter1((JSONObject) makeCall("returnsNull"));
        }
        catch (final DeserializationException e) {
            throw new RPCException(e);
        }
        catch (final Exception e) {
            dealWithException(e);
            return null;
        }
    }

    @Override
    public boolean passByReference(final ITest other, final int i) throws RPCException {

        try {
            final JSONArray args = new JSONArray();
            args.put(marshaller.serializeITest(other));
            args.put(i);
            return (Boolean) makeCall("passByReference", args).getValue();
        }
        catch (final Exception e) {
            dealWithException(e);
            return false;
        }
    }

    @Override
    public boolean isEmptyString(final String s) throws RPCException {

        try {
            final JSONArray args = new JSONArray();
            args.put(s);
            return (Boolean) makeCall("isEmptyString", args).getValue();
        }
        catch (final Exception e) {
            dealWithException(e);
            return false;
        }
    }

    @Override
    public String returnsEmptyString() throws RPCException {

        try {
            return (String) ((JSONString) makeCall("returnsEmptyString")).getValue();
        }
        catch (final DeserializationException e) {
            throw new RPCException(e);
        }
        catch (final Exception e) {
            dealWithException(e);
            return null;
        }
    }
}
