/***************************************************************************
 *                                                                         *
 * nds Library                                                             *
 * Copyright (C) 2005-2011 Distributed Systems Architecture Research Group *
 * University of St Andrews, Scotland                                      *
 * http://www-systems.cs.st-andrews.ac.uk/                                 *
 *                                                                         *
 * This file is part of nds, a package of utility classes.                 *
 *                                                                         *
 * nds is free software: you can redistribute it and/or modify             *
 * it under the terms of the GNU General Public License as published by    *
 * the Free Software Foundation, either version 3 of the License, or       *
 * (at your option) any later version.                                     *
 *                                                                         *
 * nds is distributed in the hope that it will be useful,                  *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 * GNU General Public License for more details.                            *
 *                                                                         *
 * You should have received a copy of the GNU General Public License       *
 * along with nds.  If not, see <http://www.gnu.org/licenses/>.            *
 *                                                                         *
 ***************************************************************************/
package uk.ac.standrews.cs.nds.rpc.test.app.stream;

import java.net.InetSocketAddress;

import uk.ac.standrews.cs.nds.rpc.RPCException;
import uk.ac.standrews.cs.nds.rpc.stream.Marshaller;
import uk.ac.standrews.cs.nds.rpc.stream.StreamProxy;
import uk.ac.standrews.cs.nds.rpc.test.app.ITest;
import uk.ac.standrews.cs.nds.rpc.test.app.TestException;
import uk.ac.standrews.cs.nds.rpc.test.app.TestParameter1;
import uk.ac.standrews.cs.nds.rpc.test.app.TestParameter2;

/**
 * Proxy for test service.
 * 
 * @author Graham Kirby (graham.kirby@st-andrews.ac.uk)
 */
public class TestProxy extends StreamProxy implements ITest {

    private final TestMarshaller marshaller;
    private final TestProxyImpl impl;

    // -------------------------------------------------------------------------------------------------------

    /**
     * Creates a proxy for a given server address.
     * 
     * @param node_address the address
     */
    public TestProxy(final InetSocketAddress node_address) {

        super(node_address);
        marshaller = new TestMarshaller();
        impl = new TestProxyImpl(this);
    }

    // -------------------------------------------------------------------------------------------------------

    @Override
    public Marshaller getMarshaller() {

        return marshaller;
    }

    // -------------------------------------------------------------------------------------------------------

    @Override
    public void check() throws RPCException {

        impl.check();
    }

    @Override
    public boolean equalToThree(final int i) throws RPCException {

        return impl.equalToThree(i);
    }

    @Override
    public boolean not(final boolean b) throws RPCException {

        return impl.not(b);
    }

    @Override
    public boolean xIsThree(final TestParameter1 t) throws RPCException {

        return impl.xIsThree(t);
    }

    @Override
    public boolean isEmptyString(final String s) throws RPCException {

        return impl.isEmptyString(s);
    }

    @Override
    public boolean xSameAsY(final TestParameter1 t1, final TestParameter2 t2) throws RPCException {

        return impl.xSameAsY(t1, t2);
    }

    @Override
    public boolean arraysEqual(final TestParameter2 t1, final TestParameter2 t2) throws RPCException {

        return impl.arraysEqual(t1, t2);
    }

    @Override
    public boolean allElementsHaveSameX(final TestParameter1[] t) throws RPCException {

        return impl.allElementsHaveSameX(t);
    }

    @Override
    public TestParameter1 returnsWithX3() throws RPCException {

        return impl.returnsWithX3();
    }

    @Override
    public TestParameter1[] returnsArrayLength2With5And7() throws RPCException {

        return impl.returnsArrayLength2With5And7();
    }

    @Override
    public void throwsApplicationException() throws RPCException, TestException {

        impl.throwsApplicationException();
    }

    @Override
    public void throwsNullPointerException() throws RPCException {

        impl.throwsNullPointerException();
    }

    @Override
    public TestParameter1 returnsNull() throws RPCException {

        return impl.returnsNull();
    }

    @Override
    public String returnsEmptyString() throws RPCException {

        return impl.returnsEmptyString();
    }

    @Override
    public String returnsParam(final String s) throws RPCException {

        return impl.returnsParam(s);
    }

    @Override
    public boolean passByReference(final ITest other, final int i) throws RPCException {

        return impl.passByReference(other, i);
    }
}
