/***************************************************************************
 *                                                                         *
 * nds Library                                                             *
 * Copyright (C) 2005-2011 Distributed Systems Architecture Research Group *
 * University of St Andrews, Scotland                                      *
 * http://www-systems.cs.st-andrews.ac.uk/                                 *
 *                                                                         *
 * This file is part of nds, a package of utility classes.                 *
 *                                                                         *
 * nds is free software: you can redistribute it and/or modify             *
 * it under the terms of the GNU General Public License as published by    *
 * the Free Software Foundation, either version 3 of the License, or       *
 * (at your option) any later version.                                     *
 *                                                                         *
 * nds is distributed in the hope that it will be useful,                  *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 * GNU General Public License for more details.                            *
 *                                                                         *
 * You should have received a copy of the GNU General Public License       *
 * along with nds.  If not, see <http://www.gnu.org/licenses/>.            *
 *                                                                         *
 ***************************************************************************/
package uk.ac.standrews.cs.nds.rpc.test.app.stream;

import java.net.InetSocketAddress;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.PublicKey;

import uk.ac.standrews.cs.nds.rpc.RPCException;
import uk.ac.standrews.cs.nds.rpc.security.exception.PublicKeyNotFoundLocalException;
import uk.ac.standrews.cs.nds.rpc.security.exception.PublicKeyNotFoundRemoteException;
import uk.ac.standrews.cs.nds.rpc.stream.SignableProxy;
import uk.ac.standrews.cs.nds.rpc.test.app.ISignableTest;
import uk.ac.standrews.cs.nds.rpc.test.app.ITest;
import uk.ac.standrews.cs.nds.rpc.test.app.TestException;
import uk.ac.standrews.cs.nds.rpc.test.app.TestParameter1;
import uk.ac.standrews.cs.nds.rpc.test.app.TestParameter2;

/**
 * Signed Proxy for test service.
 * 
 * @author Graham Kirby (graham.kirby@st-andrews.ac.uk)
 * @author Masih Hajiarabderkani (masih@cs.st-andrews.ac.uk)
 */
public class TestSignableProxy extends SignableProxy implements ISignableTest {

    private final TestSignableMarshaller marshaller;
    private final TestSignableProxyImpl test_impl;

    // -------------------------------------------------------------------------------------------------------

    /**
     * Creates a proxy for a given server address.
     * 
     * @param node_address the address
     * @throws NoSuchAlgorithmException the no such algorithm exception
     * @throws NoSuchProviderException the no such provider exception
     */
    public TestSignableProxy(final InetSocketAddress node_address) throws NoSuchAlgorithmException, NoSuchProviderException {

        super(node_address);

        marshaller = new TestSignableMarshaller();

        test_impl = new TestSignableProxyImpl(this);
    }

    // -------------------------------------------------------------------------------------------------------
    // SignableProxy overridden methods.

    @Override
    public TestSignableMarshaller getMarshaller() {

        return marshaller;
    }

    @Override
    public void dealWithException(final Exception e) throws RPCException {

        if (e instanceof PublicKeyNotFoundRemoteException) {

            test_impl.handlePublicKeyNotFoundRemoteException();
        }

        if (e instanceof PublicKeyNotFoundLocalException) {

            test_impl.handlePublicKeyNotFoundLocalException();
        }
        super.dealWithException(e);
    }

    // -------------------------------------------------------------------------------------------------------
    // ITest overridden methods.

    @Override
    public void check() throws RPCException {

        test_impl.check();
    }

    @Override
    public boolean equalToThree(final int i) throws RPCException {

        return test_impl.equalToThree(i);
    }

    @Override
    public boolean not(final boolean b) throws RPCException {

        return test_impl.not(b);

    }

    @Override
    public boolean xIsThree(final TestParameter1 t) throws RPCException {

        return test_impl.xIsThree(t);
    }

    @Override
    public boolean xSameAsY(final TestParameter1 t1, final TestParameter2 t2) throws RPCException {

        return test_impl.xSameAsY(t1, t2);
    }

    @Override
    public boolean arraysEqual(final TestParameter2 t1, final TestParameter2 t2) throws RPCException {

        return test_impl.arraysEqual(t1, t2);
    }

    @Override
    public boolean allElementsHaveSameX(final TestParameter1[] t) throws RPCException {

        return test_impl.allElementsHaveSameX(t);
    }

    @Override
    public TestParameter1 returnsWithX3() throws RPCException {

        return test_impl.returnsWithX3();
    }

    @Override
    public TestParameter1[] returnsArrayLength2With5And7() throws RPCException {

        return test_impl.returnsArrayLength2With5And7();
    }

    @Override
    public void throwsApplicationException() throws RPCException, TestException {

        test_impl.throwsApplicationException();
    }

    @Override
    public void throwsNullPointerException() throws RPCException {

        test_impl.throwsNullPointerException();
    }

    @Override
    public TestParameter1 returnsNull() throws RPCException {

        return test_impl.returnsNull();
    }

    @Override
    public boolean isEmptyString(final String s) throws RPCException {

        return test_impl.isEmptyString(s);
    }

    @Override
    public String returnsEmptyString() throws RPCException {

        return test_impl.returnsEmptyString();
    }

    @Override
    public String returnsParam(final String s) throws RPCException {

        return test_impl.returnsParam(s);
    }

    @Override
    public boolean passByReference(final ITest other, final int i) throws RPCException {

        return test_impl.passByReference(other, i);
    }

    // -------------------------------------------------------------------------------------------------------------------------------
    // ISignedTest overriden methods.

    @Override
    public PublicKey getPublicKey() throws RPCException {

        return test_impl.getPublicKey();
    }

    @Override
    public void notifyPublicKey(final PublicKey public_key) throws RPCException {

        test_impl.notifyPublicKey(public_key);
    }
}
