/***************************************************************************
 *                                                                         *
 * nds Library                                                             *
 * Copyright (C) 2005-2011 Distributed Systems Architecture Research Group *
 * University of St Andrews, Scotland                                      *
 * http://www-systems.cs.st-andrews.ac.uk/                                 *
 *                                                                         *
 * This file is part of nds, a package of utility classes.                 *
 *                                                                         *
 * nds is free software: you can redistribute it and/or modify             *
 * it under the terms of the GNU General Public License as published by    *
 * the Free Software Foundation, either version 3 of the License, or       *
 * (at your option) any later version.                                     *
 *                                                                         *
 * nds is distributed in the hope that it will be useful,                  *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 * GNU General Public License for more details.                            *
 *                                                                         *
 * You should have received a copy of the GNU General Public License       *
 * along with nds.  If not, see <http://www.gnu.org/licenses/>.            *
 *                                                                         *
 ***************************************************************************/
package uk.ac.standrews.cs.nds.rpc.test.app.stream;

import java.io.IOException;
import java.net.Socket;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.PublicKey;
import java.util.Set;

import org.json.JSONWriter;

import uk.ac.standrews.cs.nds.rpc.stream.IHandler;
import uk.ac.standrews.cs.nds.rpc.stream.JSONReader;
import uk.ac.standrews.cs.nds.rpc.stream.RequestHandler;

/**
 * Signed Server for test service.
 * 
 * @author Masih Hajiarabderkani (masih@cs.st-andrews.ac.uk)
 */
public class TestSignableServer extends TestServer {

    private final TestSignableImpl signable_impl;

    /**
     * Instantiates a new test signed server.
     * 
     * @param impl the impl
     * @throws NoSuchAlgorithmException the no such algorithm exception
     * @throws NoSuchProviderException the no such provider exception
     */
    public TestSignableServer(final TestSignableImpl impl) throws NoSuchAlgorithmException, NoSuchProviderException {

        super(impl);
        signable_impl = impl;

        initHandlers();
    }

    /**
     * Runs the test service.
     * 
     * @param args command line arguments including the port specified by '-p'
     * @throws Exception if the server cannot be started
     */
    public static void main(final String[] args) throws Exception {

        final TestSignableImpl impl = new TestSignableImpl();
        final TestSignableServer server = new TestSignableServer(impl);

        server.runServer(args);
    }

    @Override
    protected Runnable makeRequest(final Socket socket, final Set<RequestHandler> extant_handlers) throws IOException {

        return signable_impl.makeSignableRequestHandler(this, socket, extant_handlers);
    }

    // -------------------------------------------------------------------------------------------------------------------------------

    private void initHandlers() {

        handler_map.put("notifyPublicKey", new NotifyPublicKeyHandler());
        handler_map.put("getPublicKey", new GetPublicKeyHandler());
    }

    // -------------------------------------------------------------------------------------------------------------------------------
    // IHandler implementations

    private final class GetPublicKeyHandler implements IHandler {

        @Override
        public void execute(final JSONReader args, final JSONWriter response) throws Exception {

            marshaller.serializePublicKey(signable_impl.getPublicKey(), response);
        }
    }

    private final class NotifyPublicKeyHandler implements IHandler {

        @Override
        public void execute(final JSONReader args, final JSONWriter response) throws Exception {

            final PublicKey public_key = marshaller.deserializePublicKey(args);
            signable_impl.notifyPublicKey(public_key);
            response.value("");
        }
    }

}
