/***************************************************************************
 *                                                                         *
 * nds Library                                                             *
 * Copyright (C) 2005-2011 Distributed Systems Architecture Research Group *
 * University of St Andrews, Scotland                                      *
 * http://www-systems.cs.st-andrews.ac.uk/                                 *
 *                                                                         *
 * This file is part of nds, a package of utility classes.                 *
 *                                                                         *
 * nds is free software: you can redistribute it and/or modify             *
 * it under the terms of the GNU General Public License as published by    *
 * the Free Software Foundation, either version 3 of the License, or       *
 * (at your option) any later version.                                     *
 *                                                                         *
 * nds is distributed in the hope that it will be useful,                  *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 * GNU General Public License for more details.                            *
 *                                                                         *
 * You should have received a copy of the GNU General Public License       *
 * along with nds.  If not, see <http://www.gnu.org/licenses/>.            *
 *                                                                         *
 ***************************************************************************/
package uk.ac.standrews.cs.nds.rpc.test.nostream;

import java.lang.reflect.InvocationTargetException;
import java.net.InetSocketAddress;
import java.net.UnknownHostException;

import uk.ac.standrews.cs.nds.registry.IRegistry;
import uk.ac.standrews.cs.nds.registry.NotBoundException;
import uk.ac.standrews.cs.nds.registry.RegistryUnavailableException;
import uk.ac.standrews.cs.nds.registry.nostream.RegistryFactory;
import uk.ac.standrews.cs.nds.rpc.AbstractConnectionPool;
import uk.ac.standrews.cs.nds.rpc.RPCException;
import uk.ac.standrews.cs.nds.rpc.test.RPCTestSlowBase;
import uk.ac.standrews.cs.nds.rpc.test.app.ITest;
import uk.ac.standrews.cs.nds.rpc.test.app.nostream.TestProxy;
import uk.ac.standrews.cs.nds.rpc.test.app.nostream.TestServer;

/**
 * Tests for normal operation of non-stream-based RPC mechanism. This defines tests specific to the non-stream-based implementation, that run slowly.
 *
 * @author Graham Kirby (graham.kirby@st-andrews.ac.uk)
 */
public class RPCTestNormalOperationSlow extends RPCTestSlowBase {

    private IRegistry registry;

    @Override
    protected Class<?> getServerClass() {

        return TestServer.class;
    }

    @Override
    protected ITest bindToServer(final InetSocketAddress server_address) throws RPCException, InstantiationException, IllegalAccessException, InvocationTargetException, NoSuchMethodException {

        final TestProxy server = new TestProxy(server_address);

        server.ping();
        return server;
    }

    @Override
    protected AbstractConnectionPool getConnectionPool() {

        return TestProxy.CONNECTION_POOL;
    }

    @Override
    protected void shutdownRegistry() throws UnknownHostException, RegistryUnavailableException, RPCException, NotBoundException {

        registry.shutdown();
    }

    @Override
    public void setup() throws Exception {

        super.setup();
        registry = RegistryFactory.FACTORY.getRegistry(true);
    }
}
