/***************************************************************************
 *                                                                         *
 * nds Library                                                             *
 * Copyright (C) 2005-2011 Distributed Systems Architecture Research Group *
 * University of St Andrews, Scotland                                      *
 * http://www-systems.cs.st-andrews.ac.uk/                                 *
 *                                                                         *
 * This file is part of nds, a package of utility classes.                 *
 *                                                                         *
 * nds is free software: you can redistribute it and/or modify             *
 * it under the terms of the GNU General Public License as published by    *
 * the Free Software Foundation, either version 3 of the License, or       *
 * (at your option) any later version.                                     *
 *                                                                         *
 * nds is distributed in the hope that it will be useful,                  *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 * GNU General Public License for more details.                            *
 *                                                                         *
 * You should have received a copy of the GNU General Public License       *
 * along with nds.  If not, see <http://www.gnu.org/licenses/>.            *
 *                                                                         *
 ***************************************************************************/
package uk.ac.standrews.cs.nds.rpc.test.nostream;

import java.io.IOException;
import java.net.Socket;
import java.util.Set;

import uk.ac.standrews.cs.nds.rpc.nostream.Connection;
import uk.ac.standrews.cs.nds.rpc.nostream.RequestHandler;
import uk.ac.standrews.cs.nds.rpc.nostream.json.JSONInteger;
import uk.ac.standrews.cs.nds.rpc.nostream.json.JSONValue;

/**
 * Request handler sub-class allowing various faults for testing.
 *
 * @author Graham Kirby (graham.kirby@st-andrews.ac.uk)
 */
public class RequestHandlerFaulty extends RequestHandler {

    private static final int THREE = 3;

    private final TestServerFaulty faulty_server;

    // -------------------------------------------------------------------------------------------------------

    /**
     * Initialises a request handler.
     *
     * @param faulty_server the application server
     * @param socket the socket associated with request
     * @throws IOException if input and output streams for the socket cannot be created
     */
    public RequestHandlerFaulty(final TestServerFaulty faulty_server, final Socket socket, final Set<RequestHandler> extant_handlers) throws IOException {

        super(faulty_server, socket, extant_handlers);
        this.faulty_server = faulty_server;
    }

    // -------------------------------------------------------------------------------------------------------

    @Override
    protected void sendReply(final String method_name, final JSONValue method_result, final int id, final Connection connection) throws Exception {

        if (method_name.equals("check")) {

            if (faulty_server.dontReplyToCheck()) { return; }

            if (faulty_server.returnsInvalidJson()) {
                sendFaultyJson(connection);
                return;
            }

            if (faulty_server.returnsBlankLine()) {
                sendBlankLine(connection);
                return;
            }

            if (faulty_server.terminatesConnectionBeforeReply()) {
                try {
                    super.tearDownConnection();
                    super.tearDownSocket();
                }
                catch (final IOException e) {
                    // Ignore.
                }
                return;
            }
        }

        if (method_name.equals("equalToThree")) {

            if (faulty_server.returnsWrongType()) {
                super.sendReply(method_name, new JSONInteger(THREE), id, connection);
                return;
            }
        }

        super.sendReply(method_name, method_result, id, connection);
    }

    protected void sendFaultyJson(final Connection streams) {

        streams.println("not a valid JSON string");
    }

    protected void sendBlankLine(final Connection streams) {

        streams.println();
    }

    @Override
    protected void tearDownConnection() {

    }

    @Override
    protected void tearDownSocket() throws IOException {

    }
}
