/***************************************************************************
 *                                                                         *
 * nds Library                                                             *
 * Copyright (C) 2005-2011 Distributed Systems Architecture Research Group *
 * University of St Andrews, Scotland                                      *
 * http://www-systems.cs.st-andrews.ac.uk/                                 *
 *                                                                         *
 * This file is part of nds, a package of utility classes.                 *
 *                                                                         *
 * nds is free software: you can redistribute it and/or modify             *
 * it under the terms of the GNU General Public License as published by    *
 * the Free Software Foundation, either version 3 of the License, or       *
 * (at your option) any later version.                                     *
 *                                                                         *
 * nds is distributed in the hope that it will be useful,                  *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 * GNU General Public License for more details.                            *
 *                                                                         *
 * You should have received a copy of the GNU General Public License       *
 * along with nds.  If not, see <http://www.gnu.org/licenses/>.            *
 *                                                                         *
 ***************************************************************************/
package uk.ac.standrews.cs.nds.rpc.test.nostream;

import java.net.InetSocketAddress;

import uk.ac.standrews.cs.nds.rpc.nostream.Connection;
import uk.ac.standrews.cs.nds.rpc.nostream.json.JSONArray;
import uk.ac.standrews.cs.nds.rpc.test.app.nostream.TestProxy;

/**
 * Proxy sub-class allowing various faults for testing.
 *
 * @author Graham Kirby (graham.kirby@st-andrews.ac.uk)
 */
public class TestProxyFaulty extends TestProxy {

    private boolean send_invalid_json = false;
    private boolean send_blank_line = false;
    private boolean send_request_of_wrong_type = false;
    private boolean terminate_connection_before_request = false;
    private boolean terminate_connection_after_request = false;

    // -------------------------------------------------------------------------------------------------------

    /**
     * Creates a proxy for a given server address.
     * @param node_address the address
     */
    public TestProxyFaulty(final InetSocketAddress node_address) {

        super(node_address);
    }

    // -------------------------------------------------------------------------------------------------------

    protected void setFaultFlag(final String fault_flag) {

        if (fault_flag.equals(RPCTestErrorHandling.SEND_INVALID_JSON)) {
            send_invalid_json = true;
        }
        if (fault_flag.equals(RPCTestErrorHandling.SEND_BLANK_LINE)) {
            send_blank_line = true;
        }
        if (fault_flag.equals(RPCTestErrorHandling.SEND_REQUEST_OF_WRONG_TYPE)) {
            send_request_of_wrong_type = true;
        }
        if (fault_flag.equals(RPCTestErrorHandling.TERMINATE_CONNECTION_BEFORE_REQUEST)) {
            terminate_connection_before_request = true;
        }
        if (fault_flag.equals(RPCTestErrorHandling.TERMINATE_CONNECTION_AFTER_REQUEST)) {
            terminate_connection_after_request = true;
        }
    }

    @Override
    protected void sendRequest(final Connection streams, final String method_name, final JSONArray args) throws Exception {

        if (method_name.equals("check")) {

            if (send_invalid_json) {
                streams.println("not a valid JSON string");
                return;
            }

            if (send_blank_line) {
                streams.println();
                return;
            }

            if (terminate_connection_before_request) {
                super.releaseConnection(streams);
                return;
            }

            if (terminate_connection_after_request) {
                super.sendRequest(streams, method_name, args);
                super.closeConnection(streams);
                return;
            }
        }

        if (method_name.equals("equalToThree")) {

            if (send_request_of_wrong_type) {

                final JSONArray wrong_args = new JSONArray();
                wrong_args.put(false);
                super.sendRequest(streams, method_name, wrong_args);
                return;
            }
        }

        super.sendRequest(streams, method_name, args);
    }
}
