/***************************************************************************
 *                                                                         *
 * nds Library                                                             *
 * Copyright (C) 2005-2011 Distributed Systems Architecture Research Group *
 * University of St Andrews, Scotland                                      *
 * http://www-systems.cs.st-andrews.ac.uk/                                 *
 *                                                                         *
 * This file is part of nds, a package of utility classes.                 *
 *                                                                         *
 * nds is free software: you can redistribute it and/or modify             *
 * it under the terms of the GNU General Public License as published by    *
 * the Free Software Foundation, either version 3 of the License, or       *
 * (at your option) any later version.                                     *
 *                                                                         *
 * nds is distributed in the hope that it will be useful,                  *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 * GNU General Public License for more details.                            *
 *                                                                         *
 * You should have received a copy of the GNU General Public License       *
 * along with nds.  If not, see <http://www.gnu.org/licenses/>.            *
 *                                                                         *
 ***************************************************************************/
package uk.ac.standrews.cs.nds.rpc.test.stream;

import java.lang.reflect.InvocationTargetException;
import java.net.InetSocketAddress;
import java.net.UnknownHostException;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;

import uk.ac.standrews.cs.nds.registry.NotBoundException;
import uk.ac.standrews.cs.nds.registry.RegistryUnavailableException;
import uk.ac.standrews.cs.nds.registry.stream.RegistryFactory;
import uk.ac.standrews.cs.nds.rpc.AbstractApplicationServer;
import uk.ac.standrews.cs.nds.rpc.AbstractConnectionPool;
import uk.ac.standrews.cs.nds.rpc.RPCException;
import uk.ac.standrews.cs.nds.rpc.test.RPCTestNormalBase;
import uk.ac.standrews.cs.nds.rpc.test.app.ITest;
import uk.ac.standrews.cs.nds.rpc.test.app.stream.TestSignableImpl;
import uk.ac.standrews.cs.nds.rpc.test.app.stream.TestSignableProxy;
import uk.ac.standrews.cs.nds.rpc.test.app.stream.TestSignableServer;

/**
 * Tests for RPC mechanism.
 * 
 * @author Graham Kirby (graham.kirby@st-andrews.ac.uk)
 */
public class RPCTestSignableNormalOperation extends RPCTestNormalBase {

    private TestSignableProxy server = null;

    @Override
    protected Class<? extends TestSignableServer> getServerClass() {

        return TestSignableServer.class;
    }

    @Override
    protected AbstractApplicationServer instantiateServer() throws NoSuchAlgorithmException, NoSuchProviderException {

        return new TestSignableServer(new TestSignableImpl());
    }

    @Override
    protected ITest bindToServer(final InetSocketAddress server_address) throws RPCException, InstantiationException, IllegalAccessException, InvocationTargetException, NoSuchMethodException {

        try {
            if (server == null) {
                server = new TestSignableProxy(server_address);
            }
            server.ping();
            return server;
        }
        catch (final NoSuchAlgorithmException e) {
            e.printStackTrace();
            return null;
        }
        catch (final NoSuchProviderException e) {
            e.printStackTrace();
            return null;
        }
    }

    @Override
    protected void shutdownRegistry() throws UnknownHostException, RegistryUnavailableException, RPCException, NotBoundException {

        RegistryFactory.FACTORY.getRegistry().shutdown();
    }

    @Override
    protected AbstractConnectionPool getConnectionPool() {

        return TestSignableProxy.CONNECTION_POOL;
    }
}
