/***************************************************************************
 *                                                                         *
 * nds Library                                                             *
 * Copyright (C) 2005-2011 Distributed Systems Architecture Research Group *
 * University of St Andrews, Scotland                                      *
 * http://www-systems.cs.st-andrews.ac.uk/                                 *
 *                                                                         *
 * This file is part of nds, a package of utility classes.                 *
 *                                                                         *
 * nds is free software: you can redistribute it and/or modify             *
 * it under the terms of the GNU General Public License as published by    *
 * the Free Software Foundation, either version 3 of the License, or       *
 * (at your option) any later version.                                     *
 *                                                                         *
 * nds is distributed in the hope that it will be useful,                  *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 * GNU General Public License for more details.                            *
 *                                                                         *
 * You should have received a copy of the GNU General Public License       *
 * along with nds.  If not, see <http://www.gnu.org/licenses/>.            *
 *                                                                         *
 ***************************************************************************/
package uk.ac.standrews.cs.nds.rpc.test.stream;

import java.security.PublicKey;
import java.util.HashMap;

import uk.ac.standrews.cs.nds.rpc.security.IPublicKeyManager;
import uk.ac.standrews.cs.nds.rpc.security.exception.PublicKeyNotFoundLocalException;

/**
 * The Class TestPublicKeyManager.
 * 
 * @author Masih Hajiarabderkani (masih@cs.st-andrews.ac.uk)
 */
public class TestPublicKeyManager implements IPublicKeyManager {

    private final HashMap<String, PublicKey> public_key_map;

    /**
     * Instantiates a new test public key manager.
     */
    public TestPublicKeyManager() {

        public_key_map = new HashMap<String, PublicKey>();
    }

    @Override
    public PublicKey getPublicKey(final Object signer_id) throws PublicKeyNotFoundLocalException {

        final PublicKey result = public_key_map.get(signer_id);
        if (result == null) { throw new PublicKeyNotFoundLocalException("public key not found for id " + signer_id); }
        return result;
    }

    @Override
    public PublicKey storePublicKey(final Object signer_id, final PublicKey public_key) {

        final PublicKey put = public_key_map.put((String) signer_id, public_key);
        // System.out.println("++" + id + " STORED PUBLIC KEY for id " + signer_id + " ? " + public_key_map.get(signer_id).equals(public_key));
        return put;
    }

}
