/***************************************************************************
 *                                                                         *
 * nds Library                                                             *
 * Copyright (C) 2005-2011 Distributed Systems Architecture Research Group *
 * University of St Andrews, Scotland                                      *
 * http://www-systems.cs.st-andrews.ac.uk/                                 *
 *                                                                         *
 * This file is part of nds, a package of utility classes.                 *
 *                                                                         *
 * nds is free software: you can redistribute it and/or modify             *
 * it under the terms of the GNU General Public License as published by    *
 * the Free Software Foundation, either version 3 of the License, or       *
 * (at your option) any later version.                                     *
 *                                                                         *
 * nds is distributed in the hope that it will be useful,                  *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 * GNU General Public License for more details.                            *
 *                                                                         *
 * You should have received a copy of the GNU General Public License       *
 * along with nds.  If not, see <http://www.gnu.org/licenses/>.            *
 *                                                                         *
 ***************************************************************************/
package uk.ac.standrews.cs.nds.rpc.test.stream;

import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.PrivateKey;
import java.security.Signature;
import java.security.SignatureException;

import uk.ac.standrews.cs.nds.rpc.security.ISigner;

/**
 * The Class TestSigner.
 * 
 * @author Masih Hajiarabderkani (masih@cs.st-andrews.ac.uk)
 */
public class TestSigner implements ISigner {

    private final Signature signature;
    private final String id;
    private final PrivateKey private_key;
    private int payload_len = 0;

    /**
     * Instantiates a new test signer.
     * 
     * @param id the id
     * @param private_key the private_key
     * @param signature_algorithm the signature_algorithm
     * @param signature_algorithm_provider the signature_algorithm_provider
     * @throws NoSuchAlgorithmException the no such algorithm exception
     * @throws NoSuchProviderException the no such provider exception
     */
    public TestSigner(final String id, final PrivateKey private_key, final String signature_algorithm, final String signature_algorithm_provider) throws NoSuchAlgorithmException, NoSuchProviderException {

        this.id = id;
        this.private_key = private_key;
        signature = Signature.getInstance(signature_algorithm, signature_algorithm_provider);
    }

    @Override
    public String getId() {

        return id;
    }

    @Override
    public void initSign() throws InvalidKeyException {

        signature.initSign(private_key);
        payload_len = 0;
    }

    @Override
    public void update(final byte[] source, final int offset, final int length) throws SignatureException {

        signature.update(source, offset, length);
        payload_len += length;
    }

    @Override
    public byte[] sign() throws SignatureException {

        // System.out.println(" >> signing payload of length " + payload_len);
        return signature.sign();
    }
}
