/***************************************************************************
 *                                                                         *
 * nds Library                                                             *
 * Copyright (C) 2005-2011 Distributed Systems Architecture Research Group *
 * University of St Andrews, Scotland                                      *
 * http://www-systems.cs.st-andrews.ac.uk/                                 *
 *                                                                         *
 * This file is part of nds, a package of utility classes.                 *
 *                                                                         *
 * nds is free software: you can redistribute it and/or modify             *
 * it under the terms of the GNU General Public License as published by    *
 * the Free Software Foundation, either version 3 of the License, or       *
 * (at your option) any later version.                                     *
 *                                                                         *
 * nds is distributed in the hope that it will be useful,                  *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 * GNU General Public License for more details.                            *
 *                                                                         *
 * You should have received a copy of the GNU General Public License       *
 * along with nds.  If not, see <http://www.gnu.org/licenses/>.            *
 *                                                                         *
 ***************************************************************************/
package uk.ac.standrews.cs.nds.rpc.test.stream;

import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.PublicKey;
import java.security.Signature;
import java.security.SignatureException;

import uk.ac.standrews.cs.nds.rpc.security.IVerifier;

/**
 * The Class TestVerifier.
 *
 * @author Masih Hajiarabderkani (masih@cs.st-andrews.ac.uk)
 */
public class TestVerifier implements IVerifier {

    private final Signature signature;

    /**
     * Instantiates a new test verifier.
     *
     * @param signature_algorithm the signature_algorithm
     * @param signature_algorithm_provider the signature_algorithm_provider
     * @throws NoSuchAlgorithmException the no such algorithm exception
     * @throws NoSuchProviderException the no such provider exception
     */
    public TestVerifier(final String signature_algorithm, final String signature_algorithm_provider) throws NoSuchAlgorithmException, NoSuchProviderException {

        signature = Signature.getInstance(signature_algorithm, signature_algorithm_provider);
    }

    // -------------------------------------------------------------------------------------------------------------------------------

    @Override
    public void initVerify(final PublicKey public_key) throws InvalidKeyException {

        signature.initVerify(public_key);
    }

    @Override
    public void update(final byte[] source, final int offset, final int length) throws SignatureException {

        signature.update(source, offset, length);
    }

    @Override
    public boolean verify(final byte[] sign) throws SignatureException {

        return signature.verify(sign);
    }

}
