package uk.ac.standrews.cs.nds.rpc.test.stream;

import java.io.Reader;
import java.io.StringReader;

import junit.framework.Assert;

import org.json.JSONException;
import org.junit.Test;

import uk.ac.standrews.cs.nds.rpc.stream.JSONReader;

public class TokenizerTest {

    String t2 = "{\"params\":[{\"x\":3}],\"i\":1,\"b\":true,s:\"abc\",l=123445.00]}";

    String t3 = "{\"params\":[true],\"id\":2}";

    String t4 = "{\"params\":[{\"x\":-345987235}]}";

    String t5 = "{\"id\":0,\"result\":null}";

    String t6 = "{\"id\":0,\"result\":{\"pi\":3.14}}";

    @Test
    public void test1() throws JSONException {

        final String t1 = "{\"method\":\"foo\",\"params\":[],\"id\":0,\"jsonrpc\":\"2.0\"}";
        final Reader r = new StringReader(t1);
        final JSONReader jr = new JSONReader(r);
        jr.nextSymbol();
        jr.object();
        System.out.println(jr.key());
        System.out.println(jr.stringValue());
        System.out.println(jr.key());
        jr.array();
        jr.endArray();
        System.out.println(jr.key());
        jr.intValue();
        System.out.println(jr.key());
        System.out.println(jr.stringValue());
        System.out.println("at end");
        jr.endObject();
        System.out.println("done");
    }

    @Test
    public void test2() throws JSONException {

        final String t2 = "{\"params\":[{\"x\":3},\"b\":true,s:\"abc\",l:123445.00]}";
        final Reader r = new StringReader(t2);
        final JSONReader jr = new JSONReader(r);
        jr.nextSymbol();
        jr.object();
        System.out.println(jr.key());
        jr.array();
        jr.object();
        System.out.println(jr.key());
        System.out.println(jr.intValue());
        jr.endObject();
        System.out.println(jr.key());
        System.out.println(jr.booleanValue());
        System.out.println(jr.key());
        System.out.println(jr.stringValue());
        System.out.println(jr.key());
        System.out.println(jr.doubleValue());
        jr.endArray();
        jr.endObject();
    }

    public void parseString(final String s) throws JSONException {

        final Reader r = new StringReader(s);
        final JSONReader jr = new JSONReader(r);
        jr.nextSymbol();
        jr.object();
        System.out.println(jr.key());
        //      jr.colon();
        if (jr.have(jr.OBJECT)) {
            jr.object();
            System.out.println(jr.key());
            //       jr.colon();
            System.out.println(jr.doubleValue());
            jr.endObject();
        }
        else {
            jr.nullValue();
        }
        jr.endObject();
    }

    @Test
    public void test3() throws JSONException {

        final String t3 = "{\"result\":null}";
        final String t4 = "{\"result\":{\"pi\":3.14}}";

        parseString(t3);
        parseString(t4);
    }

    @Test
    public void test4() throws JSONException {

        final String t2 = "{\"params\":[{\"x\":3},\"b\":true,s:\"abc\",l:123445.00]}";
        final Reader r = new StringReader(t2);
        final JSONReader jr = new JSONReader(r);
        jr.nextSymbol();
        Assert.assertTrue("object", jr.have(jr.OBJECT));
        jr.object();
        Assert.assertTrue("key", jr.have(jr.STRING));
        System.out.println(jr.key());
        Assert.assertTrue("array", jr.have(jr.ARRAY));
        jr.array();
        Assert.assertTrue("object", jr.have(jr.OBJECT));
        jr.object();
        System.out.println(jr.key());
        Assert.assertTrue("int", jr.have(jr.INTEGER));
        System.out.println(jr.intValue());
        Assert.assertTrue("object", jr.have(jr.ENDOBJECT));
        jr.endObject();
        Assert.assertTrue("key", jr.have(jr.STRING));
        System.out.println(jr.key());
        Assert.assertTrue("key", jr.have(jr.BOOLEAN));
        System.out.println(jr.booleanValue());
        Assert.assertTrue("key", jr.have(jr.STRING));
        System.out.println(jr.key());
        Assert.assertTrue("key", jr.have(jr.STRING));
        System.out.println(jr.stringValue());
        Assert.assertTrue("key", jr.have(jr.STRING));
        System.out.println(jr.key());
        Assert.assertTrue("key", jr.have(jr.DOUBLE));
        System.out.println(jr.doubleValue());
        Assert.assertTrue("object", jr.have(jr.ENDARRAY));
        jr.endArray();
        Assert.assertTrue("object", jr.have(jr.ENDOBJECT));
        jr.endObject();
    }

    @Test(expected = JSONException.class)
    public void sequencewrong() throws JSONException {

        final String t1 = "{\"method\":\"foo\",\"params\":[],\"id\":0,\"jsonrpc\":\"2.0\"}";
        final Reader r = new StringReader(t1);
        final JSONReader jr = new JSONReader(r);
        jr.nextSymbol();
        jr.object();
        System.out.println(jr.key());
        System.out.println(jr.key());
        jr.array();
        jr.endArray();
        System.out.println(jr.key());
        jr.intValue();
        System.out.println(jr.key());
        System.out.println(jr.stringValue());
        System.out.println("at end");
        jr.endObject();
        System.out.println("done");
    }

    @Test(expected = JSONException.class)
    public void invalidJSON() throws JSONException {

        final String t1 = "{\"method2.0\"}";
        final Reader r = new StringReader(t1);
        final JSONReader jr = new JSONReader(r);
        jr.nextSymbol();
        jr.key();
    }
}
