package uk.ac.standrews.cs.nds.util;

import java.util.concurrent.TimeUnit;

public class Duration implements Comparable<Duration> {

    private final long length;
    private final TimeUnit unit;

    public Duration(final long length, final TimeUnit unit) {

        this.length = length;
        this.unit = unit;
    }

    public long getLength() {

        return length;
    }

    public long getLength(final TimeUnit other_time_unit) {

        return other_time_unit.convert(length, unit);
    }

    public TimeUnit getTimeUnit() {

        return unit;
    }

    public void sleep() {

        try {
            unit.sleep(length);
        }
        catch (final InterruptedException e) {
            Thread.currentThread().interrupt();
        }
    }

    public static Duration millisecondsElapsed(final long start, final long end) {

        return new Duration(end - start, TimeUnit.MILLISECONDS);
    }

    public static Duration millisecondsElapsedSince(final long start) {

        return millisecondsElapsed(start, System.currentTimeMillis());
    }

    @Override
    public int compareTo(final Duration other) {

        final long other_in_same_units = unit.convert(other.length, other.unit);
        return length > other_in_same_units ? 1 : length == other_in_same_units ? 0 : -1;
    }
}
