/***************************************************************************
 *                                                                         *
 * numonic Library                                                         *
 * Copyright (C) 2009-2011 Distributed Systems Architecture Research Group *
 * University of St Andrews, Scotland                                      *
 * http://blogs.cs.st-andrews.ac.uk/numonic                                    *
 *                                                                         *
 * This file is part of remote_management, a package providing             *
 * functionality for remotely managing a specified application.            *
 *                                                                         *
 * numonic is free software: you can redistribute it and/or                *
 * modify it under the terms of the GNU General Public License as          *
 * published by the Free Software Foundation, either version 3 of the      *
 * License, or (at your option) any later version.                         *
 *                                                                         *
 * numonic is distributed in the hope that it will be useful,              *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 * GNU General Public License for more details.                            *
 *                                                                         *
 * You should have received a copy of the GNU General Public License       *
 * along with numonic.  If not, see <http://www.gnu.org/licenses/>.        *
 *                                                                         *
 ***************************************************************************/

package uk.ac.standrews.cs.numonic.classify;

import uk.ac.standrews.cs.numonic.distribution.Distribution;

/**
 * Generic classification interface.
 * @param <T> the type of data to be classified
 *
 * @author Alan Dearle (al@cs.st-andrews.ac.uk)
 */
public interface IClassifier<T> {

    /**
     * Classifies a data item. Returns an array of probabilities, one for each of the datums being classified.
     * The arity of the array may be different from the number of datums in the data being passed in. The arity is implementation specific.
     *
     * @param data the data
     * @return an array of probabilities
     */
    Distribution[] classify(T data);
}
