package uk.ac.standrews.cs.numonic.data;

import java.util.HashMap;

/**
 * Resources that can be monitored by NUMONIC. Each enum stores a string value which is the name given to this resource in numonic.
 *
 * @author Angus Macdonald (angus AT cs.st-andrews.ac.uk)
 */
public enum ResourceType {
    //CPU Resources
    CPU_USER("cpu_user_total"), CPU_SYS("cpu_sys_total"), CPU_IDLE("cpu_idle_total"), CPU_WAIT("cpu_wait_total"), CPU_NICE("cpu_nice_total"),
    //Memory resources
    MEMORY_USED("memory_used"), MEMORY_FREE("memory_free"), SWAP_USED("swap_used"), SWAP_FREE("swap_free"),
    //Disk resources
    FS_USED("fs_space_used"), FS_FREE("fs_space_free"), FS_SIZE("fs_size"), FS_DISK_READS("fs_size"), FS_FILES("fs_number_of_files"), FS_DISK_READ_BYTES("fs_bytes_read"), FS_DISK_WRITES("fs_disk_writes"), FS_DISK_WRITE_BYTES("fs_bytes_written");

    static final HashMap<String, ResourceType> RESOURCE_MAP;

    static {
        RESOURCE_MAP = new HashMap<String, ResourceType>();

        for (final ResourceType level : ResourceType.values()) {
            RESOURCE_MAP.put(level.toString(), level);
        }
    }

    /*
     * Set-up code to store corresponding numonic value.
     */
    private final String name;

    private ResourceType(final String name) {

        this.name = name;
    }

    public static ResourceType get(final String name) {

        return RESOURCE_MAP.get(name);
    }

    @Override
    public String toString() {

        return name;
    }
}
