/***************************************************************************
 *                                                                         *
 * numonic Library                                                         *
 * Copyright (C) 2009-2011 Distributed Systems Architecture Research Group *
 * University of St Andrews, Scotland                                      *
 * http://blogs.cs.st-andrews.ac.uk/numonic                                *
 *                                                                         *
 * This file is part of numonic, a network and resource monitoring         *
 * package.                                                                *
 *                                                                         *
 * numonic is free software: you can redistribute it and/or                *
 * modify it under the terms of the GNU General Public License as          *
 * published by the Free Software Foundation, either version 3 of the      *
 * License, or (at your option) any later version.                         *
 *                                                                         *
 * numonic is distributed in the hope that it will be useful,              *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 * GNU General Public License for more details.                            *
 *                                                                         *
 * You should have received a copy of the GNU General Public License       *
 * along with numonic.  If not, see <http://www.gnu.org/licenses/>.        *
 *                                                                         *
 ***************************************************************************/

package uk.ac.standrews.cs.numonic.madface;

import java.io.File;
import java.net.InetAddress;
import java.util.ArrayList;
import java.util.List;

import uk.ac.standrews.cs.nds.madface.HostDescriptor;
import uk.ac.standrews.cs.nds.madface.JavaProcessDescriptor;
import uk.ac.standrews.cs.nds.madface.ProcessDescriptor;
import uk.ac.standrews.cs.nds.madface.interfaces.IApplicationManager;
import uk.ac.standrews.cs.nds.madface.interfaces.IGlobalHostScanner;
import uk.ac.standrews.cs.nds.madface.interfaces.ISingleHostScanner;
import uk.ac.standrews.cs.numonic.main.Numonic;
import uk.ac.standrews.cs.numonic.monitor.bandwidthAndLatency.BandwidthAndLatencyClient;

/**
 * Provides remote management hooks for numonic.
 *
 * @author Alan Dearle (al@cs.st-andrews.ac.uk)
 * @author Angus Macdonald (angus@cs.st-andrews.ac.uk)
 * @author Graham Kirby (graham.kirby@st-andrews.ac.uk)
 */
public class NumonicManager implements IApplicationManager {

    private static final String JAVA_EXECUTABLE_DIR = "/usr/java/latest/bin";

    /**
     * Path to directory containing Java executable.
     */
    public static final File JAVA_BIN_PATH = new File(JAVA_EXECUTABLE_DIR);

    /**
     * URL of properties file.
     */
    public static final String NUMONIC_PROPERTIES_URL = "http://www-systems.cs.st-andrews.ac.uk:8080/hudson/job/numonic/lastSuccessfulBuild/artifact/numonic_centralized_db_beast.properties";

    // -------------------------------------------------------------------------------------------------------

    @Override
    public String getApplicationName() {

        return "numonic";
    }

    @Override
    public void attemptApplicationCall(final HostDescriptor host_descriptor) throws Exception {

        // If this doesn't throw an exception then the application is alive.
        // This assumes that the bandwidth and latency monitoring is switched on.
        BandwidthAndLatencyClient.getLatency(InetAddress.getByName(host_descriptor.getHost()));
    }

    @Override
    public void deployApplication(final HostDescriptor host_descriptor, final Object... args) throws Exception {

        final List<String> numonic_args = new ArrayList<String>();

        numonic_args.add("-p" + NUMONIC_PROPERTIES_URL);

        host_descriptor.javaBinPath(JAVA_BIN_PATH);
        final ProcessDescriptor java_process_descriptor = new JavaProcessDescriptor().classToBeInvoked(Numonic.class).args(numonic_args);
        host_descriptor.getProcessManager().runProcess(java_process_descriptor);
    }

    @Override
    public void killApplication(final HostDescriptor host_descriptor) throws Exception {

        host_descriptor.getProcessManager().killMatchingProcesses(Numonic.class.getCanonicalName());
    }

    @Override
    public List<ISingleHostScanner> getSingleScanners() {

        return null;
    }

    @Override
    public List<IGlobalHostScanner> getGlobalScanners() {

        return null;
    }

    @Override
    public void shutdown() {

        // Nothing to shut down.
    }
}
