/***************************************************************************
 *                                                                         *
 * numonic Library                                                         *
 * Copyright (C) 2009-2011 Distributed Systems Architecture Research Group *
 * University of St Andrews, Scotland                                      *
 * http://blogs.cs.st-andrews.ac.uk/numonic                                *
 *                                                                         *
 * This file is part of numonic, a network and resource monitoring         *
 * package.                                                                *
 *                                                                         *
 * numonic is free software: you can redistribute it and/or                *
 * modify it under the terms of the GNU General Public License as          *
 * published by the Free Software Foundation, either version 3 of the      *
 * License, or (at your option) any later version.                         *
 *                                                                         *
 * numonic is distributed in the hope that it will be useful,              *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 * GNU General Public License for more details.                            *
 *                                                                         *
 * You should have received a copy of the GNU General Public License       *
 * along with numonic.  If not, see <http://www.gnu.org/licenses/>.        *
 *                                                                         *
 ***************************************************************************/

package uk.ac.standrews.cs.numonic.main.test;

import java.util.ArrayList;
import java.util.List;

import org.junit.Test;

import uk.ac.standrews.cs.nds.madface.HostDescriptor;
import uk.ac.standrews.cs.nds.madface.JavaProcessDescriptor;
import uk.ac.standrews.cs.nds.madface.ProcessDescriptor;
import uk.ac.standrews.cs.nds.madface.ProcessManager;
import uk.ac.standrews.cs.numonic.main.Numonic;
import uk.ac.standrews.cs.numonic.monitor.bandwidthAndLatency.BandwidthAndLatencyClient;

/**
 * Tests of Numonic server reporting to H2O instance.
 *
 * @author Graham Kirby (graham.kirby@st-andrews.ac.uk)
 */
public class NumonicServerTests {

    private static final String NUMONIC_CLIENT_ARG = "localhost";
    private static final String NUMONIC_SERVER_ARG = "-phttp://www-systems.cs.st-andrews.ac.uk:8080/hudson/job/numonic/lastSuccessfulBuild/artifact/numonic.properties";

    /**
     * Tests bandwidth and latency measurement within a single process.
     * @throws Exception if an error occurs
     */
    @Test
    public void singleProcessMeasurement() throws Exception {

        Numonic.main(new String[]{NUMONIC_SERVER_ARG});
        BandwidthAndLatencyClient.main(new String[]{NUMONIC_CLIENT_ARG});
    }

    /**
     * Tests bandwidth and latency measurement between two processes.
     * @throws Exception if an error occurs
     */
    @Test
    public void singleMachineMeasurement() throws Exception {

        final List<String> server_args = new ArrayList<String>();
        server_args.add(NUMONIC_SERVER_ARG);

        final List<String> client_args = new ArrayList<String>();
        client_args.add(NUMONIC_CLIENT_ARG);

        final ProcessManager process_manager = new HostDescriptor().getProcessManager();
        final ProcessDescriptor java_process_descriptor1 = new JavaProcessDescriptor().classToBeInvoked(Numonic.class).args(server_args);
        final Process p1 = process_manager.runProcess(java_process_descriptor1);
        final ProcessDescriptor java_process_descriptor2 = new JavaProcessDescriptor().classToBeInvoked(BandwidthAndLatencyClient.class).args(client_args);
        final Process p2 = process_manager.runProcess(java_process_descriptor2);

        p2.waitFor();
        p1.destroy();
    }
}
