/***************************************************************************
 *                                                                         *
 * numonic Library                                                         *
 * Copyright (C) 2009-2011 Distributed Systems Architecture Research Group *
 * University of St Andrews, Scotland                                      *
 * http://blogs.cs.st-andrews.ac.uk/numonic                                    *
 *                                                                         *
 * This file is part of remote_management, a package providing             *
 * functionality for remotely managing a specified application.            *
 *                                                                         *
 * numonic is free software: you can redistribute it and/or                *
 * modify it under the terms of the GNU General Public License as          *
 * published by the Free Software Foundation, either version 3 of the      *
 * License, or (at your option) any later version.                         *
 *                                                                         *
 * numonic is distributed in the hope that it will be useful,              *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 * GNU General Public License for more details.                            *
 *                                                                         *
 * You should have received a copy of the GNU General Public License       *
 * along with numonic.  If not, see <http://www.gnu.org/licenses/>.        *
 *                                                                         *
 ***************************************************************************/

package uk.ac.standrews.cs.numonic.monitor;

import java.util.LinkedList;
import java.util.List;

import org.hyperic.sigar.FileSystem;
import org.hyperic.sigar.FileSystemUsage;
import org.hyperic.sigar.SigarException;

import uk.ac.standrews.cs.numonic.data.FileSystemData;

/**
 * Monitor for file system data.
 *
 * @author Alan Dearle (al@cs.st-andrews.ac.uk)
 * @author Graham Kirby (graham.kirby@st-andrews.ac.uk)
 */
public class FileSystemMonitor extends MonitorBase {

    /**
     * Returns monitoring data for all local file systems.
     *
     * @return monitoring data for all local file systems
     * @throws SigarException if an error occurs enumerating the file systems or accessing their data
     */
    public List<FileSystemData> getLocalFileSystemData() throws SigarException {

        final List<FileSystemData> local_file_system_data = new LinkedList<FileSystemData>();
        final FileSystem[] file_system_list = sigar.getFileSystemList();

        for (final FileSystem file_system : file_system_list) {

            if (file_system.getType() == FileSystem.TYPE_LOCAL_DISK) {
                local_file_system_data.add(getFileSystemData(file_system));
            }
        }

        return local_file_system_data;
    }

    // -------------------------------------------------------------------------------------------------------

    /**
     * Prints out monitoring data for all local file systems.
     *
     * @param args ignored
     * @throws SigarException if an error occurs enumerating the file systems or accessing their data
     */
    public static void main(final String[] args) throws SigarException {

        final FileSystemMonitor monitor = new FileSystemMonitor();
        final List<FileSystemData> results = monitor.getLocalFileSystemData();

        for (final FileSystemData result : results) {
            System.out.println(result);
        }
    }

    // -------------------------------------------------------------------------------------------------------

    private FileSystemData getFileSystemData(final FileSystem sigar_file_system_data) throws SigarException {

        final FileSystemData data = new FileSystemData();

        data.setFileSystemLocation(sigar_file_system_data.getDevName());
        data.setFileSystemType(sigar_file_system_data.getSysTypeName());
        data.setFileSystemName(sigar_file_system_data.getDirName());

        final FileSystemUsage usage = sigar.getFileSystemUsage(data.getFileSystemName());

        data.setSpaceUsed(usage.getTotal() - usage.getFree());
        data.setSpaceFree(usage.getAvail());
        data.setSize(usage.getTotal());
        data.setNumberOfFiles(usage.getFiles());
        data.setNumberOfDiskReads(usage.getDiskReads());
        data.setBytesRead(usage.getDiskReadBytes());
        data.setNumberOfDiskWrites(usage.getDiskWrites());
        data.setBytesWritten(usage.getDiskWriteBytes());

        return data;
    }
}
