/***************************************************************************
 *                                                                         *
 * numonic Library                                                         *
 * Copyright (C) 2009-2011 Distributed Systems Architecture Research Group *
 * University of St Andrews, Scotland                                      *
 * http://blogs.cs.st-andrews.ac.uk/numonic                                    *
 *                                                                         *
 * This file is part of remote_management, a package providing             *
 * functionality for remotely managing a specified application.            *
 *                                                                         *
 * numonic is free software: you can redistribute it and/or                *
 * modify it under the terms of the GNU General Public License as          *
 * published by the Free Software Foundation, either version 3 of the      *
 * License, or (at your option) any later version.                         *
 *                                                                         *
 * numonic is distributed in the hope that it will be useful,              *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 * GNU General Public License for more details.                            *
 *                                                                         *
 * You should have received a copy of the GNU General Public License       *
 * along with numonic.  If not, see <http://www.gnu.org/licenses/>.        *
 *                                                                         *
 ***************************************************************************/

package uk.ac.standrews.cs.numonic.monitor;

import org.hyperic.sigar.SigarException;
import org.hyperic.sigar.ptql.ProcessFinder;

import uk.ac.standrews.cs.numonic.data.ProcessData;

/**
 * Monitor for H2O process.
 *
 * @author Alan Dearle (al@cs.st-andrews.ac.uk)
 */
public class H2OProcessMonitor extends ProcessMonitor {

    /**
     * Returns information on the H2O process.
     * @return information on the H2O process
     * @throws SigarException if there is an error accessing sigar data
     */
    public ProcessData getInfo() throws SigarException {

        final long processID = getH2oProcessID();

        if (processID > 0) { return getProcessData(processID); }
        return null;
    }

    /**
     * Prints out information on the H2O process.
     * @param args ignored
     * @throws SigarException if there is an error accessing sigar data
     */
    public static void main(final String[] args) throws SigarException {

        final H2OProcessMonitor process_monitor = new H2OProcessMonitor();
        final ProcessData process_data = process_monitor.getInfo();
        System.out.println(process_data);
    }

    /**
     * Returns the process ID of H2O if it is running. If it is not running, or there was an exception, 0 is returned.
     */
    private long getH2oProcessID() {

        // See: http://support.hyperic.com/display/SIGAR/PTQL
        long[] pids;
        try {
            pids = ProcessFinder.find(sigar, "State.Name.ct=java,Args.*.ct=org.h2o.H2O");

            if (pids.length != 1) { return 0; }
            return pids[0];
        }
        catch (final SigarException e) {
            // Bail out
            return 0;
        }
    }
}
