/***************************************************************************
 *                                                                         *
 * numonic Library                                                         *
 * Copyright (C) 2009-2011 Distributed Systems Architecture Research Group *
 * University of St Andrews, Scotland                                      *
 * http://blogs.cs.st-andrews.ac.uk/numonic                                    *
 *                                                                         *
 * This file is part of remote_management, a package providing             *
 * functionality for remotely managing a specified application.            *
 *                                                                         *
 * numonic is free software: you can redistribute it and/or                *
 * modify it under the terms of the GNU General Public License as          *
 * published by the Free Software Foundation, either version 3 of the      *
 * License, or (at your option) any later version.                         *
 *                                                                         *
 * numonic is distributed in the hope that it will be useful,              *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 * GNU General Public License for more details.                            *
 *                                                                         *
 * You should have received a copy of the GNU General Public License       *
 * along with numonic.  If not, see <http://www.gnu.org/licenses/>.        *
 *                                                                         *
 ***************************************************************************/

package uk.ac.standrews.cs.numonic.monitor;

import java.util.LinkedList;
import java.util.List;

import org.hyperic.sigar.NetInterfaceConfig;
import org.hyperic.sigar.NetInterfaceStat;
import org.hyperic.sigar.Sigar;
import org.hyperic.sigar.SigarException;

import uk.ac.standrews.cs.numonic.data.NetworkData;

/**
 * Monitor for file system data.
 *
 * @author Alan Dearle (al@cs.st-andrews.ac.uk)
 */
public class NetworkMonitor extends MonitorBase {

    /**
     * Returns monitoring data for the primary network device.
     *
     * @return monitoring data for the primary network device
     * @throws SigarException if an error occurs accessing sigar data
     */
    public NetworkData getPrimaryNetworkDeviceData() throws SigarException {

        return reportNetworkInterface(null);
    }

    /**
     * Returns monitoring data for all network devices.
     *
     * @return monitoring data for all network devices
     * @throws SigarException if an error occurs accessing sigar data
     */
    public List<NetworkData> getAllNetworkDeviceData() throws SigarException {

        final List<NetworkData> network_data = new LinkedList<NetworkData>();

        for (final String device_name : sigar.getNetInterfaceList()) {
            network_data.add(reportNetworkInterface(device_name));
        }

        return network_data;
    }

    // -------------------------------------------------------------------------------------------------------

    /**
     * Prints out network monitoring data for all devices.
     *
     * @param args ignored
     * @throws SigarException if an error occurs accessing sigar data
     */
    public static void main(final String[] args) throws SigarException {

        final NetworkMonitor monitor = new NetworkMonitor();
        final List<NetworkData> results = monitor.getAllNetworkDeviceData();

        for (final NetworkData result : results) {
            System.out.println(result);
        }
    }

    // -------------------------------------------------------------------------------------------------------

    // If the device name is null this will access the primary network device
    private NetworkData reportNetworkInterface(final String device_name) throws SigarException {

        final NetworkData results = new NetworkData();

        sigar = new Sigar();

        final NetInterfaceConfig ifconfig = sigar.getNetInterfaceConfig(device_name);
        final NetInterfaceStat ifstat = sigar.getNetInterfaceStat(ifconfig.getName());

        results.setDeviceName(ifconfig.getName());
        results.setIPAddress(ifconfig.getAddress());

        results.setRXBytes(ifstat.getRxBytes());
        results.setRXPackets(ifstat.getRxPackets());
        results.setRXErrors(ifstat.getRxErrors());
        results.setRXDropped(ifstat.getRxDropped());
        results.setRXOverruns(ifstat.getRxOverruns());

        results.setTXBytes(ifstat.getTxBytes());
        results.setTXPackets(ifstat.getTxPackets());
        results.setTXErrors(ifstat.getTxErrors());
        results.setTXDropped(ifstat.getTxDropped());
        results.setTXOverruns(ifstat.getTxOverruns());

        return results;
    }
}
