/***************************************************************************
 *                                                                         *
 * numonic Library                                                         *
 * Copyright (C) 2009-2011 Distributed Systems Architecture Research Group *
 * University of St Andrews, Scotland                                      *
 * http://blogs.cs.st-andrews.ac.uk/numonic                                    *
 *                                                                         *
 * This file is part of remote_management, a package providing             *
 * functionality for remotely managing a specified application.            *
 *                                                                         *
 * numonic is free software: you can redistribute it and/or                *
 * modify it under the terms of the GNU General Public License as          *
 * published by the Free Software Foundation, either version 3 of the      *
 * License, or (at your option) any later version.                         *
 *                                                                         *
 * numonic is distributed in the hope that it will be useful,              *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 * GNU General Public License for more details.                            *
 *                                                                         *
 * You should have received a copy of the GNU General Public License       *
 * along with numonic.  If not, see <http://www.gnu.org/licenses/>.        *
 *                                                                         *
 ***************************************************************************/

package uk.ac.standrews.cs.numonic.monitor;

import java.util.ArrayList;
import java.util.List;

import org.hyperic.sigar.ProcCpu;
import org.hyperic.sigar.ProcMem;
import org.hyperic.sigar.ProcUtil;
import org.hyperic.sigar.SigarException;

import uk.ac.standrews.cs.numonic.data.ProcessData;

/**
 * Monitor for process data.
 *
 * @author Alan Dearle (al@cs.st-andrews.ac.uk)
 */
public class ProcessMonitor extends MonitorBase {

    /**
     * Returns monitoring data for all processes.
     *
     * @return monitoring data for all processes
     * @throws SigarException if an error occurs accessing sigar data
     */
    public List<ProcessData> getAllProcessData() throws SigarException {

        // Find all pids.
        final long[] pids = sigar.getProcList();
        final List<ProcessData> process_data = new ArrayList<ProcessData>();

        for (final long pid : pids) {
            process_data.add(getProcessData(pid));
        }

        return process_data;
    }

    // -------------------------------------------------------------------------------------------------------

    /**
     * Prints out monitoring data for all processes.
     *
     * @param args ignored
     * @throws SigarException if an error occurs accessing sigar data
     */
    public static void main(final String[] args) throws SigarException {

        final ProcessMonitor monitor = new ProcessMonitor();
        final List<ProcessData> processes = monitor.getAllProcessData();

        for (final ProcessData result : processes) {
            System.out.println(result);
        }
    }

    // -------------------------------------------------------------------------------------------------------

    ProcessData getProcessData(final long pid) throws SigarException {

        final ProcessData results = new ProcessData();

        final ProcMem memInfo = sigar.getProcMem(pid);
        final ProcCpu cpuInfo = sigar.getProcCpu(pid);

        // TODO This is not the best - need to find more informative information.
        results.setProcessName(ProcUtil.getDescription(sigar, pid));

        // CPU.
        results.setProcessCpuPercent(cpuInfo.getPercent());
        results.setProcessStartTime(cpuInfo.getStartTime());

        // Memory.
        results.setProcessMemSize(convertBytesToKBytes(memInfo.getSize()));
        results.setProcessResident(convertBytesToKBytes(memInfo.getResident()));

        return results;
    }
}
