/***************************************************************************
 *                                                                         *
 * numonic Library                                                         *
 * Copyright (C) 2009-2011 Distributed Systems Architecture Research Group *
 * University of St Andrews, Scotland                                      *
 * http://blogs.cs.st-andrews.ac.uk/numonic                                    *
 *                                                                         *
 * This file is part of remote_management, a package providing             *
 * functionality for remotely managing a specified application.            *
 *                                                                         *
 * numonic is free software: you can redistribute it and/or                *
 * modify it under the terms of the GNU General Public License as          *
 * published by the Free Software Foundation, either version 3 of the      *
 * License, or (at your option) any later version.                         *
 *                                                                         *
 * numonic is distributed in the hope that it will be useful,              *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 * GNU General Public License for more details.                            *
 *                                                                         *
 * You should have received a copy of the GNU General Public License       *
 * along with numonic.  If not, see <http://www.gnu.org/licenses/>.        *
 *                                                                         *
 ***************************************************************************/

package uk.ac.standrews.cs.numonic.monitor;

import org.hyperic.sigar.CpuInfo;
import org.hyperic.sigar.Mem;
import org.hyperic.sigar.NetInfo;
import org.hyperic.sigar.NetInterfaceConfig;
import org.hyperic.sigar.OperatingSystem;
import org.hyperic.sigar.SigarException;
import org.hyperic.sigar.Swap;

import uk.ac.standrews.cs.numonic.data.SystemInfoData;

/**
 * Monitor for system data.
 *
 * @author Alan Dearle (al@cs.st-andrews.ac.uk)
 */
public class SystemMonitor extends MonitorBase {

    /**
     * Returns system data for the local machine.
     *
     * @param machine_id a label for the data
     * @return system data for the local machine
     * @throws SigarException if an error occurs accessing sigar data
     */
    public SystemInfoData getSystemData(final String machine_id) throws SigarException {

        // OS Info.
        final OperatingSystem sys = OperatingSystem.getInstance();

        // Network Info.
        final NetInterfaceConfig config = sigar.getNetInterfaceConfig(null);
        final NetInfo netinfo = sigar.getNetInfo();

        // CPU Info.
        final CpuInfo[] cpus = sigar.getCpuInfoList();
        final CpuInfo info = cpus[0];

        // Mem Info.
        final Mem mem = sigar.getMem();
        final Swap swap = sigar.getSwap();

        final SystemInfoData results = new SystemInfoData(machine_id, sys.getName(), sys.getVersion(), netinfo.getHostName(), config.getAddress(), netinfo.getDefaultGateway(), info.getVendor(), info.getModel(), info.getTotalCores(), info.getTotalSockets(), info.getMhz(), info.getCacheSize(),
                        convertBytesToKBytes(mem.getTotal()), convertBytesToKBytes(swap.getTotal()));

        return results;
    }

    // -------------------------------------------------------------------------------------------------------

    /**
     * Prints out local system data.
     *
     * @param args ignored
     * @throws SigarException if an error occurs accessing sigar data
     */
    public static void main(final String[] args) throws SigarException {

        final SystemMonitor monitor = new SystemMonitor();
        System.out.println(monitor.getSystemData(""));
    }
}
