/***************************************************************************
 *                                                                         *
 * numonic Library                                                         *
 * Copyright (C) 2009-2011 Distributed Systems Architecture Research Group *
 * University of St Andrews, Scotland                                      *
 * http://blogs.cs.st-andrews.ac.uk/numonic                                    *
 *                                                                         *
 * This file is part of remote_management, a package providing             *
 * functionality for remotely managing a specified application.            *
 *                                                                         *
 * numonic is free software: you can redistribute it and/or                *
 * modify it under the terms of the GNU General Public License as          *
 * published by the Free Software Foundation, either version 3 of the      *
 * License, or (at your option) any later version.                         *
 *                                                                         *
 * numonic is distributed in the hope that it will be useful,              *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 * GNU General Public License for more details.                            *
 *                                                                         *
 * You should have received a copy of the GNU General Public License       *
 * along with numonic.  If not, see <http://www.gnu.org/licenses/>.        *
 *                                                                         *
 ***************************************************************************/
package uk.ac.standrews.cs.numonic.reportingFactories;

import uk.ac.standrews.cs.numonic.db.DatabaseConnection;
import uk.ac.standrews.cs.numonic.reporting.IReporting;
import uk.ac.standrews.cs.numonic.util.IPropertiesWrapper;

/**
 * Factory for creating an H2O DatabaseConnection.
 *
 * @author Alan Dearle (al@cs.st-andrews.ac.uk)
 */
public class DatabaseConnectionFactory implements IReportingFactory {

    private static final String DATABASEURL = "databaseURL";
    private static final String USERNAME = "username";
    private static final String PASSWORD = "password";

    private String databaseURL;
    private String username;
    private String password;

    private boolean initialised = false;

    @Override
    public void init(final IPropertiesWrapper properties) throws ReportingInitialisationException {

        databaseURL = getProperty(properties, DATABASEURL);
        username = getProperty(properties, USERNAME);
        password = getProperty(properties, PASSWORD);

        initialised = true;
    }

    @Override
    public IReporting create() throws ReportingInitialisationException {

        if (!initialised) { throw new ReportingInitialisationException("init must be called before create"); }
        return new DatabaseConnection(databaseURL, username, password, false);
    }

    private String getProperty(final IPropertiesWrapper properties, final String key) throws ReportingInitialisationException {

        final String property = properties.getProperty(key);
        if (property == null) { throw new ReportingInitialisationException("No property specified for: " + key); }
        return property;
    }
}
