package uk.ac.standrews.cs.numonic.util;

import java.io.IOException;
import java.util.Set;

public interface IPropertiesWrapper {

    /**
     * Load the properties file into memory.
     *
     * @throws IOException if the properties file does not exist or couldn't be loaded.
     */
    public abstract void loadProperties() throws IOException;

    /**
     * Deletes any existing properties file with the given name and creates a new one.
     *
     * @throws IOException if an error occurs while creating a file
     */
    public abstract void createNewFile() throws IOException;

    /**
     * Gets the property associated with a given key
     *
     * @param key a key
     * @return the corresponding property
     */
    public abstract String getProperty(final String key);

    /**
     * Tests whether the property for a given key is true.
     *
     * @param key a key
     * @return true if the property for the key is true
     */
    public abstract boolean isEnabled(final String key);

    /**
     * Sets the property for a given key, and saves to the backing file.
     *
     * @param key a key
     * @param value the new value to be associated with the key
     */
    public abstract void setProperty(final String key, final String value);

    /**
     * Sets the property comment.
     *
     * @param propertyComment the comment
     */
    public abstract void setPropertyComment(final String propertyComment);

    /**
     * Save the properties file and close it.
     *
     * @throws IOException if the file couldn't be saved.
     */
    public abstract void saveAndClose() throws IOException;

    /**
     * Returns a set of the keys in this properties file.
     *
     * @return a set of the keys in this properties file
     */
    public abstract Set<Object> getKeys();

}
